/*
ssc (static site checker)
Copyright (c) 2020,2021 Dylan Harris
https://dylanharris.org/

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public Licence as published by
the Free Software Foundation, either version 3 of the Licence, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public Licence for more details.

You should have received a copy of the GNU General Public
Licence along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "main/standard.h"
#include "schema/schema_name.h"
#include "utility/common.h"

struct property_name
{   const char* name_ = nullptr;
    e_microdata_root root_;
    e_schema_property prop_;
#ifdef REQUIRE_CONSTRUCTOR
    property_name ( const char* name, const e_microdata_root root, const e_schema_property prop)
        : name_ (name), root_ (root), prop_ (prop) { }
#endif // REQUIRE_CONSTRUCTOR
};

property_name namtab [] =
{   { "about", mdr_schema, sp_about },
    { "abridged", mdr_schema, sp_abridged },
    { "abstract", mdr_schema, sp_abstract },
    { "accelerationTime", mdr_schema, sp_accelerationtime },
    { "acceptedAnswer", mdr_schema, sp_acceptedanswer },
    { "acceptedOffer", mdr_schema, sp_acceptedoffer },
    { "acceptedPaymentMethod", mdr_schema, sp_acceptedpaymentmethod },
    { "acceptsReservations", mdr_schema, sp_acceptsreservations },
    { "accessCode", mdr_schema, sp_accesscode },
    { "accessMode", mdr_schema, sp_accessmode },
    { "accessModeSufficient", mdr_schema, sp_accessmodesufficient },
    { "accessibilityAPI", mdr_schema, sp_accessibilityapi },
    { "accessibilityControl", mdr_schema, sp_accessibilitycontrol },
    { "accessibilityFeature", mdr_schema, sp_accessibilityfeature },
    { "accessibilityHazard", mdr_schema, sp_accessibilityhazard },
    { "accessibilitySummary", mdr_schema, sp_accessibilitysummary },
    { "accommodationCategory", mdr_schema, sp_accommodationcategory },
    { "accommodationFloorPlan", mdr_schema, sp_accommodationfloorplan },
    { "accountId", mdr_schema, sp_accountid },
    { "accountMinimumInflow", mdr_schema, sp_accountminimuminflow },
    { "accountOverdraftLimit", mdr_schema, sp_accountoverdraftlimit },
    { "accountablePerson", mdr_schema, sp_accountableperson },
    { "acquireLicensePage", mdr_schema, sp_acquirelicensepage },
    { "acquiredFrom", mdr_schema, sp_acquiredfrom },
    { "acrissCode", mdr_schema, sp_acrisscode },
    { "action", mdr_schema, sp_action },
    { "actionAccessibilityRequirement", mdr_schema, sp_actionaccessibilityrequirement },
    { "actionApplication", mdr_schema, sp_actionapplication },
    { "actionOption", mdr_schema, sp_actionoption },
    { "actionPlatform", mdr_schema, sp_actionplatform },
    { "actionStatus", mdr_schema, sp_actionstatus },
    { "actionableFeedbackPolicy", mdr_schema, sp_actionablefeedbackpolicy },
    { "activeIngredient", mdr_schema, sp_activeingredient },
    { "activityDuration", mdr_schema, sp_activityduration },
    { "activityFrequency", mdr_schema, sp_activityfrequency },
    { "actor", mdr_schema, sp_actor },
    { "addOn", mdr_schema, sp_addon },
    { "additionalName", mdr_schema, sp_additionalname },
    { "additionalNumberOfGuests", mdr_schema, sp_additionalnumberofguests },
    { "additionalProperty", mdr_schema, sp_additionalproperty },
    { "additionalType", mdr_schema, sp_additionaltype },
    { "additionalVariable", mdr_schema, sp_additionalvariable },
    { "address", mdr_schema, sp_address },
    { "addressCountry", mdr_schema, sp_addresscountry },
    { "addressLocality", mdr_schema, sp_addresslocality },
    { "addressRegion", mdr_schema, sp_addressregion },
    { "administrationRoute", mdr_schema, sp_administrationroute },
    { "advanceBookingRequirement", mdr_schema, sp_advancebookingrequirement },
    { "adverseOutcome", mdr_schema, sp_adverseoutcome },
    { "affectedBy", mdr_schema, sp_affectedby },
    { "affiliation", mdr_schema, sp_affiliation },
    { "afterMedia", mdr_schema, sp_aftermedia },
    { "agent", mdr_schema, sp_agent },
    { "aggregateRating", mdr_schema, sp_aggregaterating },
    { "aircraft", mdr_schema, sp_aircraft },
    { "album", mdr_schema, sp_album },
    { "albumProductionType", mdr_schema, sp_albumproductiontype },
    { "albumRelease", mdr_schema, sp_albumrelease },
    { "albumReleaseType", mdr_schema, sp_albumreleasetype },
    { "alcoholWarning", mdr_schema, sp_alcoholwarning },
    { "algorithm", mdr_schema, sp_algorithm },
    { "alignmentType", mdr_schema, sp_alignmenttype },
    { "alternateName", mdr_schema, sp_alternatename },
    { "alternativeHeadline", mdr_schema, sp_alternativeheadline },
    { "alumni", mdr_schema, sp_alumni },
    { "alumniOf", mdr_schema, sp_alumniof },
    { "amenityFeature", mdr_schema, sp_amenityfeature },
    { "amount", mdr_schema, sp_amount },
    { "amountOfThisGood", mdr_schema, sp_amountofthisgood },
    { "announcementLocation", mdr_schema, sp_announcementlocation },
    { "annualPercentageRate", mdr_schema, sp_annualpercentagerate },
    { "answerCount", mdr_schema, sp_answercount },
    { "antagonist", mdr_schema, sp_antagonist },
    { "appearance", mdr_schema, sp_appearance },
    { "applicableLocation", mdr_schema, sp_applicablelocation },
    { "applicantContact", mdr_schema, sp_applicantcontact },
    { "applicantLocationRequirements", mdr_schema, sp_applicantlocationrequirements },
    { "applicationCategory", mdr_schema, sp_applicationcategory },
    { "applicationDeadline", mdr_schema, sp_applicationdeadline },
    { "applicationStartDate", mdr_schema, sp_applicationstartdate },
    { "applicationSubCategory", mdr_schema, sp_applicationsubcategory },
    { "applicationSuite", mdr_schema, sp_applicationsuite },
    { "appliesToDeliveryMethod", mdr_schema, sp_appliestodeliverymethod },
    { "appliesToPaymentMethod", mdr_schema, sp_appliestopaymentmethod },
    { "archiveHeld", mdr_schema, sp_archiveheld },
    { "areaServed", mdr_schema, sp_areaserved },
    { "arrivalAirport", mdr_schema, sp_arrivalairport },
    { "arrivalBoatTerminal", mdr_schema, sp_arrivalboatterminal },
    { "arrivalBusStop", mdr_schema, sp_arrivalbusstop },
    { "arrivalGate", mdr_schema, sp_arrivalgate },
    { "arrivalPlatform", mdr_schema, sp_arrivalplatform },
    { "arrivalStation", mdr_schema, sp_arrivalstation },
    { "arrivalTerminal", mdr_schema, sp_arrivalterminal },
    { "arrivalTime", mdr_schema, sp_arrivaltime },
    { "artEdition", mdr_schema, sp_artedition },
    { "artMedium", mdr_schema, sp_artmedium },
    { "arterialBranch", mdr_schema, sp_arterialbranch },
    { "artform", mdr_schema, sp_artform },
    { "articleBody", mdr_schema, sp_articlebody },
    { "articleSection", mdr_schema, sp_articlesection },
    { "artist", mdr_schema, sp_artist },
    { "artworkSurface", mdr_schema, sp_artworksurface },
    { "aspect", mdr_schema, sp_aspect },
    { "assemblyVersion", mdr_schema, sp_assemblyversion },
    { "assess", mdr_schema, sp_assess },
    { "assesses", mdr_schema, sp_assesses },
    { "associatedAnatomy", mdr_schema, sp_associatedanatomy },
    { "associatedArticle", mdr_schema, sp_associatedarticle },
    { "associatedMedia", mdr_schema, sp_associatedmedia },
    { "associatedPathophysiology", mdr_schema, sp_associatedpathophysiology },
    { "athlete", mdr_schema, sp_athlete },
    { "attendee", mdr_schema, sp_attendee },
    { "audience", mdr_schema, sp_audience },
    { "audienceType", mdr_schema, sp_audiencetype },
    { "audio", mdr_schema, sp_audio },
    { "authenticator", mdr_schema, sp_authenticator },
    { "author", mdr_schema, sp_author },
    { "availabeDeliveryMethod", mdr_schema, sp_availabedeliverymethod },
    { "availability", mdr_schema, sp_availability },
    { "availabilityEnds", mdr_schema, sp_availabilityends },
    { "availabilityStarts", mdr_schema, sp_availabilitystarts },
    { "availableAtOrFrom", mdr_schema, sp_availableatorfrom },
    { "availableChannel", mdr_schema, sp_availablechannel },
    { "availableDeliveryMethod", mdr_schema, sp_availabledeliverymethod },
    { "availableFrom", mdr_schema, sp_availablefrom },
    { "availableIn", mdr_schema, sp_availablein },
    { "availableLanguage", mdr_schema, sp_availablelanguage },
    { "availableLocation", mdr_schema, sp_availablelocation },
    { "availableOnDevice", mdr_schema, sp_availableondevice },
    { "availableService", mdr_schema, sp_availableservice },
    { "availableStrength", mdr_schema, sp_availablestrength },
    { "availableTest", mdr_schema, sp_availabletest },
    { "availableThrough", mdr_schema, sp_availablethrough },
    { "award", mdr_schema, sp_award },
    { "awayTeam", mdr_schema, sp_awayteam },
    { "background", mdr_schema, sp_background },
    { "backstory", mdr_schema, sp_backstory },
    { "bankAccountType", mdr_schema, sp_bankaccounttype },
    { "baseSalary", mdr_schema, sp_basesalary },
    { "bccRecipient", mdr_schema, sp_bccrecipient },
    { "bed", mdr_schema, sp_bed },
    { "beforeMedia", mdr_schema, sp_beforemedia },
    { "beneficiaryBank", mdr_schema, sp_beneficiarybank },
    { "benefitsSummaryUrl", mdr_schema, sp_benefitssummaryurl },
    { "bestRating", mdr_schema, sp_bestrating },
    { "billingAddress", mdr_schema, sp_billingaddress },
    { "billingDuration", mdr_schema, sp_billingduration },
    { "billingIncrement", mdr_schema, sp_billingincrement },
    { "billingPeriod", mdr_schema, sp_billingperiod },
    { "billingStart", mdr_schema, sp_billingstart },
    { "biomechanicalClass", mdr_schema, sp_biomechanicalclass },
    { "birthDate", mdr_schema, sp_birthdate },
    { "birthPlace", mdr_schema, sp_birthplace },
    { "bitrate", mdr_schema, sp_bitrate },
    { "blogPost", mdr_schema, sp_blogpost },
    { "bloodSupply", mdr_schema, sp_bloodsupply },
    { "boardingGroup", mdr_schema, sp_boardinggroup },
    { "boardingPolicy", mdr_schema, sp_boardingpolicy },
    { "bodyLocation", mdr_schema, sp_bodylocation },
    { "bodyType", mdr_schema, sp_bodytype },
    { "bookEdition", mdr_schema, sp_bookedition },
    { "bookFormat", mdr_schema, sp_bookformat },
    { "bookingTime", mdr_schema, sp_bookingtime },
    { "box", mdr_schema, sp_box },
    { "branch", mdr_schema, sp_branch },
    { "branchCode", mdr_schema, sp_branchcode },
    { "brand", mdr_schema, sp_brand },
    { "breadcrumb", mdr_schema, sp_breadcrumb },
    { "breastfeedingWarning", mdr_schema, sp_breastfeedingwarning },
    { "broadcastAffiliateOf", mdr_schema, sp_broadcastaffiliateof },
    { "broadcastChannelId", mdr_schema, sp_broadcastchannelid },
    { "broadcastDisplayName", mdr_schema, sp_broadcastdisplayname },
    { "broadcastFrequency", mdr_schema, sp_broadcastfrequency },
    { "broadcastFrequencyValue", mdr_schema, sp_broadcastfrequencyvalue },
    { "broadcastOfEvent", mdr_schema, sp_broadcastofevent },
    { "broadcastServiceTier", mdr_schema, sp_broadcastservicetier },
    { "broadcastSignalModulation", mdr_schema, sp_broadcastsignalmodulation },
    { "broadcastSubChannel", mdr_schema, sp_broadcastsubchannel },
    { "broadcastTimezone", mdr_schema, sp_broadcasttimezone },
    { "broadcaster", mdr_schema, sp_broadcaster },
    { "broker", mdr_schema, sp_broker },
    { "browserRequirements", mdr_schema, sp_browserrequirements },
    { "busName", mdr_schema, sp_busname },
    { "busNumber", mdr_schema, sp_busnumber },
    { "businessDays", mdr_schema, sp_businessdays },
    { "businessFunction", mdr_schema, sp_businessfunction },
    { "buyer", mdr_schema, sp_buyer },
    { "byArtist", mdr_schema, sp_byartist },
    { "byDay", mdr_schema, sp_byday },
    { "byMonth", mdr_schema, sp_bymonth },
    { "byMonthDay", mdr_schema, sp_bymonthday },
    { "byMonthWeek", mdr_schema, sp_bymonthweek },
    { "callSign", mdr_schema, sp_callsign },
    { "calories", mdr_schema, sp_calories },
    { "candidate", mdr_schema, sp_candidate },
    { "caption", mdr_schema, sp_caption },
    { "carbohydrateContent", mdr_schema, sp_carbohydratecontent },
    { "cargoVolume", mdr_schema, sp_cargovolume },
    { "carrierRequirements", mdr_schema, sp_carrierrequirements },
    { "cashBack", mdr_schema, sp_cashback },
    { "catalogNumber", mdr_schema, sp_cataloguenumber },
    { "category", mdr_schema, sp_category },
    { "cause", mdr_schema, sp_cause },
    { "causeOf", mdr_schema, sp_causeof },
    { "ccRecipient", mdr_schema, sp_ccrecipient },
    { "character", mdr_schema, sp_character },
    { "characterAttribute", mdr_schema, sp_characterattribute },
    { "characterName", mdr_schema, sp_charactername },
    { "cheatCode", mdr_schema, sp_cheatcode },
    { "checkinTime", mdr_schema, sp_checkintime },
    { "checkoutTime", mdr_schema, sp_checkouttime },
    { "childMaxAge", mdr_schema, sp_childmaxage },
    { "childMinAge", mdr_schema, sp_childminage },
    { "children", mdr_schema, sp_children },
    { "cholesterolContent", mdr_schema, sp_cholesterolcontent },
    { "circle", mdr_schema, sp_circle },
    { "citation", mdr_schema, sp_citation },
    { "claimReviewed", mdr_schema, sp_claimreviewed },
    { "clinicalPharamcology", mdr_schema, sp_clinicalpharamcology },
    { "clipNumber", mdr_schema, sp_clipnumber },
    { "closes", mdr_schema, sp_closes },
    { "coach", mdr_schema, sp_coach },
    { "code", mdr_schema, sp_code },
    { "codeRepository", mdr_schema, sp_coderepository },
    { "codeSampleType", mdr_schema, sp_codesampletype },
    { "codeValue", mdr_schema, sp_codevalue },
    { "codingSystem", mdr_schema, sp_codingsystem },
    { "colleague", mdr_schema, sp_colleague },
    { "collectionSize", mdr_schema, sp_collectionsize },
    { "color", mdr_schema, sp_colour },
    { "colorist", mdr_schema, sp_colourist },
    { "comment", mdr_schema, sp_comment },
    { "commentCount", mdr_schema, sp_commentcount },
    { "commentText", mdr_schema, sp_commenttext },
    { "commentTime", mdr_schema, sp_commenttime },
    { "competencyRequired", mdr_schema, sp_competencyrequired },
    { "competitor", mdr_schema, sp_competitor },
    { "composer", mdr_schema, sp_composer },
    { "comprisedOf", mdr_schema, sp_comprisedof },
    { "conditionsOfAccess", mdr_schema, sp_conditionsofaccess },
    { "confirmationNumber", mdr_schema, sp_confirmationnumber },
    { "connectedTo", mdr_schema, sp_connectedto },
    { "constrainingProperty", mdr_schema, sp_constrainingproperty },
    { "contactOption", mdr_schema, sp_contactoption },
    { "contactPoint", mdr_schema, sp_contactpoint },
    { "contactType", mdr_schema, sp_contacttype },
    { "contactlessPayment", mdr_schema, sp_contactlesspayment },
    { "containedIn", mdr_schema, sp_containedin },
    { "containedInPlace", mdr_schema, sp_containedinplace },
    { "containsPlace", mdr_schema, sp_containsplace },
    { "containsSeason", mdr_schema, sp_containsseason },
    { "contentLocation", mdr_schema, sp_contentlocation },
    { "contentRating", mdr_schema, sp_contentrating },
    { "contentReferenceTime", mdr_schema, sp_contentreferencetime },
    { "contentSize", mdr_schema, sp_contentsize },
    { "contentType", mdr_schema, sp_contenttype },
    { "contentUrl", mdr_schema, sp_contenturl },
    { "contraindication", mdr_schema, sp_contraindication },
    { "contributor", mdr_schema, sp_contributor },
    { "cookTime", mdr_schema, sp_cooktime },
    { "cookingMethod", mdr_schema, sp_cookingmethod },
    { "copyrightHolder", mdr_schema, sp_copyrightholder },
    { "copyrightNotice", mdr_schema, sp_copyrightnotice },
    { "copyrightYear", mdr_schema, sp_copyrightyear },
    { "correction", mdr_schema, sp_correction },
    { "correctionsPolicy", mdr_schema, sp_correctionspolicy },
    { "cost", mdr_schema, sp_cost },
    { "costCategory", mdr_schema, sp_costcategory },
    { "costCurrency", mdr_schema, sp_costcurrency },
    { "costOrigin", mdr_schema, sp_costorigin },
    { "costPerUnit", mdr_schema, sp_costperunit },
    { "countriesNotSupported", mdr_schema, sp_countriesnotsupported },
    { "countriesSupported", mdr_schema, sp_countriessupported },
    { "countryOfOrigin", mdr_schema, sp_countryoforigin },
    { "courseCode", mdr_schema, sp_coursecode },
    { "courseMode", mdr_schema, sp_coursemode },
    { "coursePresequisites", mdr_schema, sp_coursepresequisites },
    { "courseWorkload", mdr_schema, sp_courseworkload },
    { "coverageEndTime", mdr_schema, sp_coverageendtime },
    { "coverageStartTime", mdr_schema, sp_coveragestarttime },
    { "creativeWorkStatus", mdr_schema, sp_creativeworkstatus },
    { "creator", mdr_schema, sp_creator },
    { "credentialCategory", mdr_schema, sp_credentialcategory },
    { "creditText", mdr_schema, sp_credittext },
    { "creditedTo", mdr_schema, sp_creditedto },
    { "cssSelector", mdr_schema, sp_cssselector },
    { "currenciesAccepted", mdr_schema, sp_currenciesaccepted },
    { "currency", mdr_schema, sp_currency },
    { "currentExchangeRate", mdr_schema, sp_currentexchangerate },
    { "customer", mdr_schema, sp_customer },
    { "cutoffTime", mdr_schema, sp_cutofftime },
    { "cvdCollectionDate", mdr_schema, sp_cvdcollectiondate },
    { "cvdFacilityCounty", mdr_schema, sp_cvdfacilitycounty },
    { "cvdFacilityId", mdr_schema, sp_cvdfacilityid },
    { "cvdNumBeds", mdr_schema, sp_cvdnumbeds },
    { "cvdNumBedsOcc", mdr_schema, sp_cvdnumbedsocc },
    { "cvdNumC19Died", mdr_schema, sp_cvdnumc19died },
    { "cvdNumC19HOPats", mdr_schema, sp_cvdnumc19hopats },
    { "cvdNumC19HospPats", mdr_schema, sp_cvdnumc19hosppats },
    { "cvdNumC19MechVentPats", mdr_schema, sp_cvdnumc19mechventpats },
    { "cvdNumC19OFMechVentPats", mdr_schema, sp_cvdnumc19ofmechventpats },
    { "cvdNumC19OverflowPats", mdr_schema, sp_cvdnumc19overflowpats },
    { "cvdNumICUBeds", mdr_schema, sp_cvdnumicubeds },
    { "cvdNumICUBedsOcc", mdr_schema, sp_cvdnumicubedsocc },
    { "cvdNumTotBeds", mdr_schema, sp_cvdnumtotbeds },
    { "cvdNumVent", mdr_schema, sp_cvdnumvent },
    { "cvdNumVentUse", mdr_schema, sp_cvdnumventuse },
    { "dataFeedElement", mdr_schema, sp_datafeedelement },
    { "dataset", mdr_schema, sp_dataset },
    { "datasetTimeInterval", mdr_schema, sp_datasettimeinterval },
    { "dateCreated", mdr_schema, sp_datecreated },
    { "dateDeleted", mdr_schema, sp_datedeleted },
    { "dateIssued", mdr_schema, sp_dateissued },
    { "dateModified", mdr_schema, sp_datemodified },
    { "datePosted", mdr_schema, sp_dateposted },
    { "datePublished", mdr_schema, sp_datepublished },
    { "dateRead", mdr_schema, sp_dateread },
    { "dateReceived", mdr_schema, sp_datereceived },
    { "dateSent", mdr_schema, sp_datesent },
    { "dateVehicleFirstRegistered", mdr_schema, sp_datevehiclefirstregistered },
    { "dateline", mdr_schema, sp_dateline },
    { "dayOfWeek", mdr_schema, sp_dayofweek },
    { "deathDate", mdr_schema, sp_deathdate },
    { "deathPlace", mdr_schema, sp_deathplace },
    { "defaultValue", mdr_schema, sp_defaultvalue },
    { "deliveryAddress", mdr_schema, sp_deliveryaddress },
    { "deliveryLeadTime", mdr_schema, sp_deliveryleadtime },
    { "deliveryMethod", mdr_schema, sp_deliverymethod },
    { "deliveryStatus", mdr_schema, sp_deliverystatus },
    { "deliveryTime", mdr_schema, sp_deliverytime },
    { "department", mdr_schema, sp_department },
    { "departureAirport", mdr_schema, sp_departureairport },
    { "departureBoatTerminal", mdr_schema, sp_departureboatterminal },
    { "departureBusStop", mdr_schema, sp_departurebusstop },
    { "departureGate", mdr_schema, sp_departuregate },
    { "departurePlatform", mdr_schema, sp_departureplatform },
    { "departureStation", mdr_schema, sp_departurestation },
    { "departureTerminal", mdr_schema, sp_departureterminal },
    { "departureTime", mdr_schema, sp_departuretime },
    { "dependencies", mdr_schema, sp_dependencies },
    { "depth", mdr_schema, sp_depth },
    { "description", mdr_schema, sp_description },
    { "diagnosis", mdr_schema, sp_diagnosis },
    { "diagram", mdr_schema, sp_diagram },
    { "diet", mdr_schema, sp_diet },
    { "dietFeatures", mdr_schema, sp_dietfeatures },
    { "differentialDiagnosis", mdr_schema, sp_differentialdiagnosis },
    { "director", mdr_schema, sp_director },
    { "disambiguatingDescription", mdr_schema, sp_disambiguatingdescription },
    { "discount", mdr_schema, sp_discount },
    { "discountCode", mdr_schema, sp_discountcode },
    { "discountCurrency", mdr_schema, sp_discountcurrency },
    { "discusses", mdr_schema, sp_discusses },
    { "discussionUrl", mdr_schema, sp_discussionurl },
    { "diseasePreventionInfo", mdr_schema, sp_diseasepreventioninfo },
    { "diseaseSpreadStatistics", mdr_schema, sp_diseasespreadstatistics },
    { "dissolutionDate", mdr_schema, sp_dissolutiondate },
    { "distance", mdr_schema, sp_distance },
    { "distinguishingSign", mdr_schema, sp_distinguishingsign },
    { "distribution", mdr_schema, sp_distribution },
    { "diversityPolicy", mdr_schema, sp_diversitypolicy },
    { "diversityStaffingReport", mdr_schema, sp_diversitystaffingreport },
    { "documentation", mdr_schema, sp_documentation },
    { "doesNotShip", mdr_schema, sp_doesnotship },
    { "domainIncludes", mdr_schema, sp_domainincludes },
    { "domiciledMortgage", mdr_schema, sp_domiciledmortgage },
    { "doorTime", mdr_schema, sp_doortime },
    { "dosageForm", mdr_schema, sp_dosageform },
    { "doseSchedule", mdr_schema, sp_doseschedule },
    { "doseUnit", mdr_schema, sp_doseunit },
    { "doseValue", mdr_schema, sp_dosevalue },
    { "downPayment", mdr_schema, sp_downpayment },
    { "downloadUrl", mdr_schema, sp_downloadurl },
    { "downvoteCount", mdr_schema, sp_downvotecount },
    { "drainsTo", mdr_schema, sp_drainsto },
    { "driveWheelConfiguration", mdr_schema, sp_drivewheelconfiguration },
    { "dropoffLocation", mdr_schema, sp_dropofflocation },
    { "dropoffTime", mdr_schema, sp_dropofftime },
    { "drug", mdr_schema, sp_drug },
    { "drugClass", mdr_schema, sp_drugclass },
    { "drugUnit", mdr_schema, sp_drugunit },
    { "duns", mdr_schema, sp_duns },
    { "duplicateTherapy", mdr_schema, sp_duplicatetherapy },
    { "duration", mdr_schema, sp_duration },
    { "durationOfWarranty", mdr_schema, sp_durationofwarranty },
    { "duringMedia", mdr_schema, sp_duringmedia },
    { "earlyPrepaymentPenalty", mdr_schema, sp_earlyprepaymentpenalty },
    { "editEIDR", mdr_schema, sp_editeidr },
    { "editor", mdr_schema, sp_editor },
    { "eduQuestionType", mdr_schema, sp_eduquestiontype },
    { "educationCredentialAwarded", mdr_schema, sp_educationcredentialawarded },
    { "educationRequirements", mdr_schema, sp_educationrequirements },
    { "educationalAlignment", mdr_schema, sp_educationalalignment },
    { "educationalCredentialAwarded", mdr_schema, sp_educationalcredentialawarded },
    { "educationalFramework", mdr_schema, sp_educationalframework },
    { "educationalLevel", mdr_schema, sp_educationallevel },
    { "educationalProgramMode", mdr_schema, sp_educationalprogrammode },
    { "educationalRole", mdr_schema, sp_educationalrole },
    { "educationalUse", mdr_schema, sp_educationaluse },
    { "elevation", mdr_schema, sp_elevation },
    { "eligibileCustomerType", mdr_schema, sp_eligibilecustomertype },
    { "eligibilityToWorkRequirement", mdr_schema, sp_eligibilitytoworkrequirement },
    { "eligibleCustomerType", mdr_schema, sp_eligiblecustomertype },
    { "eligibleDuration", mdr_schema, sp_eligibleduration },
    { "eligibleQuantity", mdr_schema, sp_eligiblequantity },
    { "eligibleRegion", mdr_schema, sp_eligibleregion },
    { "eligibleTransactionVolume", mdr_schema, sp_eligibletransactionvolume },
    { "email", mdr_schema, sp_email },
    { "embedUrl", mdr_schema, sp_embedurl },
    { "emmissionsCO2", mdr_schema, sp_emmissionsco2 },
    { "employee", mdr_schema, sp_employee },
    { "employerOverview", mdr_schema, sp_employeroverview },
    { "employmentType", mdr_schema, sp_employmenttype },
    { "employmentUnit", mdr_schema, sp_employmentunit },
    { "encodesCreativeWork", mdr_schema, sp_encodescreativework },
    { "encoding", mdr_schema, sp_encoding },
    { "encodingFormat", mdr_schema, sp_encodingformat },
    { "encodingType", mdr_schema, sp_encodingtype },
    { "endDate", mdr_schema, sp_enddate },
    { "endOffset", mdr_schema, sp_endoffset },
    { "endTime", mdr_schema, sp_endtime },
    { "endorsee", mdr_schema, sp_endorsee },
    { "endorsers", mdr_schema, sp_endorsers },
    { "energyEfficiencyScaleMax", mdr_schema, sp_energyefficiencyscalemax },
    { "energyEfficiencyScaleMin", mdr_schema, sp_energyefficiencyscalemin },
    { "engineDisplacement", mdr_schema, sp_enginedisplacement },
    { "enginePower", mdr_schema, sp_enginepower },
    { "engineType", mdr_schema, sp_enginetype },
    { "entertainmentBusiness", mdr_schema, sp_entertainmentbusiness },
    { "epidemiology", mdr_schema, sp_epidemiology },
    { "episode", mdr_schema, sp_episode },
    { "episodeNumber", mdr_schema, sp_episodenumber },
    { "equal", mdr_schema, sp_equal },
    { "error", mdr_schema, sp_error },
    { "estimatedCost", mdr_schema, sp_estimatedcost },
    { "estimatedFlightDuration", mdr_schema, sp_estimatedflightduration },
    { "estimatedSalary", mdr_schema, sp_estimatedsalary },
    { "estimatesRiskOf", mdr_schema, sp_estimatesriskof },
    { "ethicsPolicy", mdr_schema, sp_ethicspolicy },
    { "event", mdr_schema, sp_event },
    { "eventAttendanceMode", mdr_schema, sp_eventattendancemode },
    { "eventSchedule", mdr_schema, sp_eventschedule },
    { "eventStatus", mdr_schema, sp_eventstatus },
    { "evidenceLevel", mdr_schema, sp_evidencelevel },
    { "evidenceOrigin", mdr_schema, sp_evidenceorigin },
    { "exampleOfWork", mdr_schema, sp_exampleofwork },
    { "exceptDate", mdr_schema, sp_exceptdate },
    { "exchangeRateSpread", mdr_schema, sp_exchangeratespread },
    { "executableLibraryName", mdr_schema, sp_executablelibraryname },
    { "exerciseCourse", mdr_schema, sp_exercisecourse },
    { "exercisePlan", mdr_schema, sp_exerciseplan },
    { "exerciseRelatedDiet", mdr_schema, sp_exerciserelateddiet },
    { "exerciseType", mdr_schema, sp_exercisetype },
    { "exifData", mdr_schema, sp_exifdata },
    { "expectedArrivalFrom", mdr_schema, sp_expectedarrivalfrom },
    { "expectedArrivalUntil", mdr_schema, sp_expectedarrivaluntil },
    { "expectedPrognosis", mdr_schema, sp_expectedprognosis },
    { "expectsAcceptanceOf", mdr_schema, sp_expectsacceptanceof },
    { "experienceRequirements", mdr_schema, sp_experiencerequirements },
    { "expeienceRequirements", mdr_schema, sp_experiencerequirements }, // typo in early original schema.org docs
    { "experienceInPlaceOfEducation", mdr_schema, sp_experienceinplaceofeducation },
    { "expires", mdr_schema, sp_expires },
    { "exportConsiderations", mdr_schema, sp_exportconsiderations },
    { "familyName", mdr_schema, sp_familyname },
    { "fatContent", mdr_schema, sp_fatcontent },
    { "faxNumber", mdr_schema, sp_faxnumber },
    { "featureList", mdr_schema, sp_featurelist },
    { "feesAndCommissionsSpecification", mdr_schema, sp_feesandcommissionsspecification },
    { "fiberContent", mdr_schema, sp_fibrecontent },
    { "fileFormat", mdr_schema, sp_fileformat },
    { "fileSize", mdr_schema, sp_filesize },
    { "financialAidEligible", mdr_schema, sp_financialaideligible },
    { "firstAppearance", mdr_schema, sp_firstappearance },
    { "firstPerformance", mdr_schema, sp_firstperformance },
    { "flightDistance", mdr_schema, sp_flightdistance },
    { "flightNumber", mdr_schema, sp_flightnumber },
    { "floorLevel", mdr_schema, sp_floorlevel },
    { "floorLimit", mdr_schema, sp_floorlimit },
    { "floorSize", mdr_schema, sp_floorsize },
    { "followee", mdr_schema, sp_followee },
    { "follows", mdr_schema, sp_follows },
    { "followup", mdr_schema, sp_followup },
    { "foodEstablishment", mdr_schema, sp_foodestablishment },
    { "foodEvent", mdr_schema, sp_foodevent },
    { "foodWarning", mdr_schema, sp_foodwarning },
    { "founder", mdr_schema, sp_founder },
    { "foundingDate", mdr_schema, sp_foundingdate },
    { "foundingLocation", mdr_schema, sp_foundinglocation },
    { "freeShippingThreshold", mdr_schema, sp_freeshippingthreshold },
    { "frequency", mdr_schema, sp_frequency },
    { "fromLocation", mdr_schema, sp_fromlocation },
    { "fuelCapacity", mdr_schema, sp_fuelcapacity },
    { "fuelConsumption", mdr_schema, sp_fuelconsumption },
    { "fuelEfficiency", mdr_schema, sp_fuelefficiency },
    { "fuelType", mdr_schema, sp_fueltype },
    { "funcionalClass", mdr_schema, sp_funcionalclass },
    { "function", mdr_schema, sp_function },
    { "fundedItem", mdr_schema, sp_fundeditem },
    { "funder", mdr_schema, sp_funder },
    { "game", mdr_schema, sp_game },
    { "gameItem", mdr_schema, sp_gameitem },
    { "gameLocation", mdr_schema, sp_gamelocation },
    { "gamePlatform", mdr_schema, sp_gameplatform },
    { "gameServer", mdr_schema, sp_gameserver },
    { "gameTip", mdr_schema, sp_gametip },
    { "gender", mdr_schema, sp_gender },
    { "genre", mdr_schema, sp_genre },
    { "geo", mdr_schema, sp_geo },
    { "geoContains", mdr_schema, sp_geocontains },
    { "geoCoveredBy", mdr_schema, sp_geocoveredby },
    { "geoCovers", mdr_schema, sp_geocovers },
    { "geoCrosses", mdr_schema, sp_geocrosses },
    { "geoDisjoint", mdr_schema, sp_geodisjoint },
    { "geoEquals", mdr_schema, sp_geoequals },
    { "geoIntersects", mdr_schema, sp_geointersects },
    { "geoMidpoint", mdr_schema, sp_geomidpoint },
    { "geoOverlaps", mdr_schema, sp_geooverlaps },
    { "geoRadius", mdr_schema, sp_georadius },
    { "geoTouches", mdr_schema, sp_geotouches },
    { "geoWithin", mdr_schema, sp_geowithin },
    { "geographicArea", mdr_schema, sp_geographicarea },
    { "gettingTestedInfo", mdr_schema, sp_gettingtestedinfo },
    { "givenName", mdr_schema, sp_givenname },
    { "globalLocationNumber", mdr_schema, sp_globallocationnumber },
    { "governmentBenefitsInfo", mdr_schema, sp_governmentbenefitsinfo },
    { "gracePeriod", mdr_schema, sp_graceperiod },
    { "grantee", mdr_schema, sp_grantee },
    { "greater", mdr_schema, sp_greater },
    { "greaterOrEqual", mdr_schema, sp_greaterorequal },
    { "gtin", mdr_schema, sp_gtin },
    { "gtin12", mdr_schema, sp_gtin12 },
    { "gtin13", mdr_schema, sp_gtin13 },
    { "gtin14", mdr_schema, sp_gtin14 },
    { "gtin8", mdr_schema, sp_gtin8 },
    { "guideline", mdr_schema, sp_guideline },
    { "guidelineDate", mdr_schema, sp_guidelinedate },
    { "guidelineSubject", mdr_schema, sp_guidelinesubject },
    { "handlingTime", mdr_schema, sp_handlingtime },
    { "hasBroadcastChannel", mdr_schema, sp_hasbroadcastchannel },
    { "hasCategoryCode", mdr_schema, sp_hascategorycode },
    { "hasCourse", mdr_schema, sp_hascourse },
    { "hasCourseInstance", mdr_schema, sp_hascourseinstance },
    { "hasCredential", mdr_schema, sp_hascredential },
    { "hasDefinedTerm", mdr_schema, sp_hasdefinedterm },
    { "hasDeliveryMethod", mdr_schema, sp_hasdeliverymethod },
    { "hasDigitalDocumentPermission", mdr_schema, sp_hasdigitaldocumentpermission },
    { "hasDriveThroughService", mdr_schema, sp_hasdrivethroughservice },
    { "hasEnergyConsumptionDetails", mdr_schema, sp_hasenergyconsumptiondetails },
    { "hasEnergyEfficiencyCategory", mdr_schema, sp_hasenergyefficiencycategory },
    { "hasHealthAspect", mdr_schema, sp_hashealthaspect },
    { "hasMap", mdr_schema, sp_hasmap },
    { "hasMeasurement", mdr_schema, sp_hasmeasurement },
    { "hasMenu", mdr_schema, sp_hasmenu },
    { "hasMenuItem", mdr_schema, sp_hasmenuitem },
    { "hasMenuSection", mdr_schema, sp_hasmenusection },
    { "hasMerchantReturnPolicy", mdr_schema, sp_hasmerchantreturnpolicy },
    { "hasOccupation", mdr_schema, sp_hasoccupation },
    { "hasOfferCatalog", mdr_schema, sp_hasoffercatalogue },
    { "hasPOS", mdr_schema, sp_haspos },
    { "hasPart", mdr_schema, sp_haspart },
    { "hasProductReturnPolicy", mdr_schema, sp_hasproductreturnpolicy },
    { "hasVariant", mdr_schema, sp_hasvariant },
    { "headline", mdr_schema, sp_headline },
    { "healthCondition", mdr_schema, sp_healthcondition },
    { "healthPlanCoinsuranceOption", mdr_schema, sp_healthplancoinsuranceoption },
    { "healthPlanCoinsuranceRate", mdr_schema, sp_healthplancoinsurancerate },
    { "healthPlanCopay", mdr_schema, sp_healthplancopay },
    { "healthPlanCopayOption", mdr_schema, sp_healthplancopayoption },
    { "healthPlanCostSharing", mdr_schema, sp_healthplancostsharing },
    { "healthPlanDrugOption", mdr_schema, sp_healthplandrugoption },
    { "healthPlanDrugTier", mdr_schema, sp_healthplandrugtier },
    { "healthPlanId", mdr_schema, sp_healthplanid },
    { "healthPlanMarketingUrl", mdr_schema, sp_healthplanmarketingurl },
    { "healthPlanNetworkId", mdr_schema, sp_healthplannetworkid },
    { "healthPlanNetworkTier", mdr_schema, sp_healthplannetworktier },
    { "healthPlanPharmacyCategory", mdr_schema, sp_healthplanpharmacycategory },
    { "healthcareReportingData", mdr_schema, sp_healthcarereportingdata },
    { "height", mdr_schema, sp_height },
    { "highPrice", mdr_schema, sp_highprice },
    { "hiringOrganization", mdr_schema, sp_hiringorganization },
    { "holdingArchive", mdr_schema, sp_holdingarchive },
    { "homeLocation", mdr_schema, sp_homelocation },
    { "homeTeam", mdr_schema, sp_hometeam },
    { "honorificPrefix", mdr_schema, sp_honourificprefix },
    { "honorificSuffix", mdr_schema, sp_honourificsuffix },
    { "hospitalAffiliation", mdr_schema, sp_hospitalaffiliation },
    { "hostingOrganization", mdr_schema, sp_hostingorganisation },
    { "hoursAvailable", mdr_schema, sp_hoursavailable },
    { "howPerformed", mdr_schema, sp_howperformed },
    { "httpMethod", mdr_schema, sp_httpmethod },
    { "iataCode", mdr_schema, sp_iatacode },
    { "identifier", mdr_schema, sp_identifier },
    { "identifyingExam", mdr_schema, sp_identifyingexam },
    { "identifyingTest", mdr_schema, sp_identifyingtest },
    { "illustrator", mdr_schema, sp_illustrator },
    { "image", mdr_schema, sp_image },
    { "imagingTechnique", mdr_schema, sp_imagingtechnique },
    { "inAlbum", mdr_schema, sp_inalbum },
    { "inBroadcastLineup", mdr_schema, sp_inbroadcastlineup },
    { "inCodeSet", mdr_schema, sp_incodeset },
    { "inDefinedTermSet", mdr_schema, sp_indefinedtermset },
    { "inLanguage", mdr_schema, sp_inlanguage },
    { "inPlaylist", mdr_schema, sp_inplaylist },
    { "inProductGroupWithID", mdr_schema, sp_inproductgroupwithid },
    { "inStoreReturnsOffered", mdr_schema, sp_instorereturnsoffered },
    { "inSupportOf", mdr_schema, sp_insupportof },
    { "incentiveCompensation", mdr_schema, sp_incentivecompensation },
    { "includedComposition", mdr_schema, sp_includedcomposition },
    { "includedDataCatalog", mdr_schema, sp_includeddatacatalog },
    { "includedInHealthInsurancePlan", mdr_schema, sp_includedinhealthinsuranceplan },
    { "includedRiskFactor", mdr_schema, sp_includedriskfactor },
    { "includesAttraction", mdr_schema, sp_includesattraction },
    { "includesHealthPlanFormulary", mdr_schema, sp_includeshealthplanformulary },
    { "includesHealthPlanNetwork", mdr_schema, sp_includeshealthplannetwork },
    { "includesObject", mdr_schema, sp_includesobject },
    { "increasesRiskOf", mdr_schema, sp_increasesriskof },
    { "indication", mdr_schema, sp_indication },
    { "industry", mdr_schema, sp_industry },
    { "ineligibleRegion", mdr_schema, sp_ineligibleregion },
    { "infectiousAgent", mdr_schema, sp_infectiousagent },
    { "infectiousAgentClass", mdr_schema, sp_infectiousagentclass },
    { "inker", mdr_schema, sp_inker },
    { "insertion", mdr_schema, sp_insertion },
    { "installUrl", mdr_schema, sp_installurl },
    { "instructor", mdr_schema, sp_instructor },
    { "instrument", mdr_schema, sp_instrument },
    { "intensity", mdr_schema, sp_intensity },
    { "interactingDrug", mdr_schema, sp_interactingdrug },
    { "interactionService", mdr_schema, sp_interactionservice },
    { "interactionStatistic", mdr_schema, sp_interactionstatistic },
    { "interactionType", mdr_schema, sp_interactiontype },
    { "interactivityType", mdr_schema, sp_interactivitytype },
    { "interestRate", mdr_schema, sp_interestrate },
    { "inventoryLevel", mdr_schema, sp_inventorylevel },
    { "inverseOf", mdr_schema, sp_inverseof },
    { "isAcceptingNewPatients", mdr_schema, sp_isacceptingnewpatients },
    { "isAccessibleForFree", mdr_schema, sp_isaccessibleforfree },
    { "isAccessoryOrSparePartFor", mdr_schema, sp_isaccessoryorsparepartfor },
    { "isAvailableGenerally", mdr_schema, sp_isavailablegenerally },
    { "isBasedOn", mdr_schema, sp_isbasedon },
    { "isBasedOnUrl", mdr_schema, sp_isbasedonurl },
    { "isConsumableFor", mdr_schema, sp_isconsumablefor },
    { "isFamilyFriendly", mdr_schema, sp_isfamilyfriendly },
    { "isGift", mdr_schema, sp_isgift },
    { "isLiveBroadcast", mdr_schema, sp_islivebroadcast },
    { "isPartOf", mdr_schema, sp_ispartof },
    { "isPlanForApartment", mdr_schema, sp_isplanforapartment },
    { "isProprietary", mdr_schema, sp_isproprietary },
    { "isRelatedTo", mdr_schema, sp_isrelatedto },
    { "isResizable", mdr_schema, sp_isresizable },
    { "isSimilarTo", mdr_schema, sp_issimilarto },
    { "isUnlabelledFallback", mdr_schema, sp_isunlabelledfallback },
    { "isVariantOf", mdr_schema, sp_isvariantof },
    { "isbn", mdr_schema, sp_isbn },
    { "isicV4", mdr_schema, sp_isicv4 },
    { "isrcCode", mdr_schema, sp_isrccode },
    { "issn", mdr_schema, sp_issn },
    { "issueNumber", mdr_schema, sp_issuenumber },
    { "issuedBy", mdr_schema, sp_issuedby },
    { "issuedThrough", mdr_schema, sp_issuedthrough },
    { "iswcCode", mdr_schema, sp_iswccode },
    { "item", mdr_schema, sp_item },
    { "itemCondition", mdr_schema, sp_itemcondition },
    { "itemListElement", mdr_schema, sp_itemlistelement },
    { "itemListOrder", mdr_schema, sp_itemlistorder },
    { "itemLocation", mdr_schema, sp_itemlocation },
    { "itemOffered", mdr_schema, sp_itemoffered },
    { "itemReviewed", mdr_schema, sp_itemreviewed },
    { "itemShipped", mdr_schema, sp_itemshipped },
    { "itinerary", mdr_schema, sp_itinerary },
    { "jobBenefits", mdr_schema, sp_jobbenefits },
    { "jobImmediateStart", mdr_schema, sp_jobimmediatestart },
    { "jobLocation", mdr_schema, sp_joblocation },
    { "jobLocationType", mdr_schema, sp_joblocationtype },
    { "jobStartDate", mdr_schema, sp_jobstartdate },
    { "jobTitle", mdr_schema, sp_jobtitle },
    { "jurisdiction", mdr_schema, sp_jurisdiction },
    { "keywords", mdr_schema, sp_keywords },
    { "knownVehicleDamages", mdr_schema, sp_knownvehicledamages },
    { "knows", mdr_schema, sp_knows },
    { "knowsAbout", mdr_schema, sp_knowsabout },
    { "knowsLanguage", mdr_schema, sp_knowslanguage },
    { "labelDetails", mdr_schema, sp_labeldetails },
    { "landlord", mdr_schema, sp_landlord },
    { "lastReviewed", mdr_schema, sp_lastreviewed },
    { "latitude", mdr_schema, sp_latitude },
    { "layoutImage", mdr_schema, sp_layoutimage },
    { "learningResourceType", mdr_schema, sp_learningresourcetype },
    { "leaseLength", mdr_schema, sp_leaselength },
    { "legalName", mdr_schema, sp_legalname },
    { "legalStatus", mdr_schema, sp_legalstatus },
    { "legislationApplies", mdr_schema, sp_legislationapplies },
    { "legislationChanges", mdr_schema, sp_legislationchanges },
    { "legislationConsolidates", mdr_schema, sp_legislationconsolidates },
    { "legislationDate", mdr_schema, sp_legislationdate },
    { "legislationDateVersion", mdr_schema, sp_legislationdateversion },
    { "legislationIdentifier", mdr_schema, sp_legislationidentifier },
    { "legislationJurisdiction", mdr_schema, sp_legislationjurisdiction },
    { "legislationLegalForce", mdr_schema, sp_legislationlegalforce },
    { "legislationLegalValue", mdr_schema, sp_legislationlegalvalue },
    { "legislationPassedBy", mdr_schema, sp_legislationpassedby },
    { "legislationResponsible", mdr_schema, sp_legislationresponsible },
    { "legislationTransposes", mdr_schema, sp_legislationtransposes },
    { "legislationType", mdr_schema, sp_legislationtype },
    { "leiCode", mdr_schema, sp_leicode },
    { "lender", mdr_schema, sp_lender },
    { "lesser", mdr_schema, sp_lesser },
    { "lesserOrEqual", mdr_schema, sp_lesserorequal },
    { "letterer", mdr_schema, sp_letterer },
    { "license", mdr_schema, sp_license },
    { "line", mdr_schema, sp_line },
    { "linkRelationship", mdr_schema, sp_linkrelationship },
    { "liveBlogUpdate", mdr_schema, sp_liveblogupdate },
    { "loanMortgagaMandateAmount", mdr_schema, sp_loanmortgagamandateamount },
    { "loanPaymentAmount", mdr_schema, sp_loanpaymentamount },
    { "loanPaymentFrequency", mdr_schema, sp_loanpaymentfrequency },
    { "loanRepaymentForm", mdr_schema, sp_loanrepaymentform },
    { "loanTerm", mdr_schema, sp_loanterm },
    { "loanType", mdr_schema, sp_loantype },
    { "location", mdr_schema, sp_location },
    { "locationCreated", mdr_schema, sp_locationcreated },
    { "lodgingUnitDescription", mdr_schema, sp_lodgingunitdescription },
    { "lodgingUnitType", mdr_schema, sp_lodgingunittype },
    { "logo", mdr_schema, sp_logo },
    { "longitude", mdr_schema, sp_longitude },
    { "loser", mdr_schema, sp_loser },
    { "lowPrice", mdr_schema, sp_lowprice },
    { "lyricist", mdr_schema, sp_lyricist },
    { "lyrics", mdr_schema, sp_lyrics },
    { "mainContentOfPage", mdr_schema, sp_maincontentofpage },
    { "mainEntity", mdr_schema, sp_mainentity },
    { "mainEntityOfPage", mdr_schema, sp_mainentityofpage },
    { "maintainer", mdr_schema, sp_maintainer },
    { "makesOffer", mdr_schema, sp_makesoffer },
    { "manufacturer", mdr_schema, sp_manufacturer },
    { "mapType", mdr_schema, sp_maptype },
    { "marginOfError", mdr_schema, sp_marginoferror },
    { "masthead", mdr_schema, sp_masthead },
    { "material", mdr_schema, sp_material },
    { "materialExtent", mdr_schema, sp_materialextent },
    { "mathExpression", mdr_schema, sp_mathexpression },
    { "maxPrice", mdr_schema, sp_maxprice },
    { "maxValue", mdr_schema, sp_maxvalue },
    { "maximumAttendeeCapacity", mdr_schema, sp_maximumattendeecapacity },
    { "maximumEnrollment", mdr_schema, sp_maximumenrollment },
    { "maximumIntake", mdr_schema, sp_maximumintake },
    { "maximumPhysicalAttendeeCapacity", mdr_schema, sp_maximumphysicalattendeecapacity },
    { "maximumVirtualAttendeeCapacity", mdr_schema, sp_maximumvirtualattendeecapacity },
    { "mealService", mdr_schema, sp_mealservice },
    { "measuredProperty", mdr_schema, sp_measuredproperty },
    { "measuredValue", mdr_schema, sp_measuredvalue },
    { "measurementTechnique", mdr_schema, sp_measurementtechnique },
    { "mechanismOfAction", mdr_schema, sp_mechanismofaction },
    { "mediaAuthenticityCategory", mdr_schema, sp_mediaauthenticitycategory },
    { "median", mdr_schema, sp_median },
//    { "medicalAudience", mdr_schema, sp_aspect },
    { "medicalAudience", mdr_schema, sp_medicalaudience },
    { "medicalSpeciality", mdr_schema, sp_medicalspeciality },
    { "medicineSystem", mdr_schema, sp_medicinesystem },
    { "meetsEmissionStandard", mdr_schema, sp_meetsemissionstandard },
    { "member", mdr_schema, sp_member },
    { "memberOf", mdr_schema, sp_memberof },
    { "membershipNumber", mdr_schema, sp_membershipnumber },
    { "membershipPointsEarned", mdr_schema, sp_membershippointsearned },
    { "memoryRequirements", mdr_schema, sp_memoryrequirements },
    { "mentions", mdr_schema, sp_mentions },
    { "menu", mdr_schema, sp_menu },
    { "menuAddOn", mdr_schema, sp_menuaddon },
    { "merchantReturnDays", mdr_schema, sp_merchantreturndays },
    { "merchantReturnLink", mdr_schema, sp_merchantreturnlink },
    { "messageAttachment", mdr_schema, sp_messageattachment },
    { "mileageFromOdometer", mdr_schema, sp_mileagefromodometer },
    { "minPrice", mdr_schema, sp_minprice },
    { "minValue", mdr_schema, sp_minvalue },
    { "minimumMonthlyRepaymentAmount", mdr_schema, sp_minimummonthlyrepaymentamount },
    { "minimumPaymentDue", mdr_schema, sp_minimumpaymentdue },
    { "missionCoveragePrioritiesPolicy", mdr_schema, sp_missioncoverageprioritiespolicy },
    { "model", mdr_schema, sp_model },
    { "modelDate", mdr_schema, sp_modeldate },
    { "modifiedTime", mdr_schema, sp_modifiedtime },
    { "monthsOfExperience", mdr_schema, sp_monthsofexperience },
    { "mpn", mdr_schema, sp_mpn },
    { "multipleValues", mdr_schema, sp_multiplevalues },
    { "muscleAction", mdr_schema, sp_muscleaction },
    { "musicArrangement", mdr_schema, sp_musicarrangement },
    { "musicBy", mdr_schema, sp_musicby },
    { "musicCompositionForm", mdr_schema, sp_musiccompositionform },
    { "musicReleaseFormat", mdr_schema, sp_musicreleaseformat },
    { "musicalKey", mdr_schema, sp_musicalkey },
    { "naics", mdr_schema, sp_naics },
    { "name", mdr_schema, sp_name },
    { "nationality", mdr_schema, sp_nationality },
    { "naturalProgression", mdr_schema, sp_naturalprogression },
    { "nerve", mdr_schema, sp_nerve },
    { "nerveMotor", mdr_schema, sp_nervemotor },
    { "netWorth", mdr_schema, sp_networth },
    { "newUpdatesAndGuideLines", mdr_schema, sp_newupdatesandguidelines },
    { "nextItem", mdr_schema, sp_nextitem },
    { "noBylinesPolicy", mdr_schema, sp_nobylinespolicy },
    { "nonEqual", mdr_schema, sp_nonequal },
    { "nonPropriatryName", mdr_schema, sp_nonpropriatryname },
    { "nonProprietaryName", mdr_schema, sp_nonproprietaryname },
    { "nonprofitStatus", mdr_schema, sp_nonprofitstatus },
    { "normalRange", mdr_schema, sp_normalrange },
    { "nsn", mdr_schema, sp_nsn },
    { "numAdults", mdr_schema, sp_numadults },
    { "numChildren", mdr_schema, sp_numchildren },
    { "numConstraints", mdr_schema, sp_numconstraints },
    { "numTracks", mdr_schema, sp_numtracks },
    { "numberOfAccommodationUnits", mdr_schema, sp_numberofaccommodationunits },
    { "numberOfAirbags", mdr_schema, sp_numberofairbags },
    { "numberOfAvailableAccommodationUnits", mdr_schema, sp_numberofavailableaccommodationunits },
    { "numberOfAxles", mdr_schema, sp_numberofaxles },
    { "numberOfBathroomsTotal", mdr_schema, sp_numberofbathroomstotal },
    { "numberOfBedrooms", mdr_schema, sp_numberofbedrooms },
    { "numberOfBeds", mdr_schema, sp_numberofbeds },
    { "numberOfCredits", mdr_schema, sp_numberofcredits },
    { "numberOfDoors", mdr_schema, sp_numberofdoors },
    { "numberOfEmployees", mdr_schema, sp_numberofemployees },
    { "numberOfEpisodes", mdr_schema, sp_numberofepisodes },
    { "numberOfForwardgears", mdr_schema, sp_numberofforwardgears },
    { "numberOfFullBathrooms", mdr_schema, sp_numberoffullbathrooms },
    { "numberOfItems", mdr_schema, sp_numberofitems },
    { "numberOfLoanPayments", mdr_schema, sp_numberofloanpayments },
    { "numberOfPages", mdr_schema, sp_numberofpages },
    { "numberOfPartialBathrooms", mdr_schema, sp_numberofpartialbathrooms },
    { "numberOfPlayers", mdr_schema, sp_numberofplayers },
    { "numberOfPreviousowners", mdr_schema, sp_numberofpreviousowners },
    { "numberOfRooms", mdr_schema, sp_numberofrooms },
    { "numberOfSeasons", mdr_schema, sp_numberofseasons },
    { "numberedPosition", mdr_schema, sp_numberedposition },
    { "nutrition", mdr_schema, sp_nutrition },
    { "object", mdr_schema, sp_object },
    { "observationDate", mdr_schema, sp_observationdate },
    { "observedNode", mdr_schema, sp_observednode },
    { "ocaoCode", mdr_schema, sp_ocaocode },
    { "occupancy", mdr_schema, sp_occupancy },
    { "occupationLocation", mdr_schema, sp_occupationlocation },
    { "occupationalCategory", mdr_schema, sp_occupationalcategory },
    { "occupationalCredentialAwarded", mdr_schema, sp_occupationalcredentialawarded },
    { "offerCount", mdr_schema, sp_offercount },
    { "offeredBy", mdr_schema, sp_offeredby },
    { "offers", mdr_schema, sp_offers },
    { "offersPrescriptionByMail", mdr_schema, sp_offersprescriptionbymail },
    { "openingHours", mdr_schema, sp_openinghours },
    { "openingHoursSpecification", mdr_schema, sp_openinghoursspecification },
    { "opens", mdr_schema, sp_opens },
    { "operatingSystem", mdr_schema, sp_operatingsystem },
    { "opponent", mdr_schema, sp_opponent },
    { "orderDate", mdr_schema, sp_orderdate },
    { "orderDelivery", mdr_schema, sp_orderdelivery },
    { "orderItemNumber", mdr_schema, sp_orderitemnumber },
    { "orderItemStatus", mdr_schema, sp_orderitemstatus },
    { "orderNumber", mdr_schema, sp_ordernumber },
    { "orderQuantity", mdr_schema, sp_orderquantity },
    { "orderStatus", mdr_schema, sp_orderstatus },
    { "orderedItem", mdr_schema, sp_ordereditem },
    { "organizer", mdr_schema, sp_organiser },
    { "origin", mdr_schema, sp_origin },
    { "originAddress", mdr_schema, sp_originaddress },
    { "originatesFrom", mdr_schema, sp_originatesfrom },
    { "outcome", mdr_schema, sp_outcome },
    { "overdosage", mdr_schema, sp_overdosage },
    { "overview", mdr_schema, sp_overview },
    { "ownedFrom", mdr_schema, sp_ownedfrom },
    { "ownedThrough", mdr_schema, sp_ownedthrough },
    { "ownershipFundingInfo", mdr_schema, sp_ownershipfundinginfo },
    { "owns", mdr_schema, sp_owns },
    { "pageEnd", mdr_schema, sp_pageend },
    { "pageStart", mdr_schema, sp_pagestart },
    { "pagination", mdr_schema, sp_pagination },
    { "parent", mdr_schema, sp_parent },
    { "parentItem", mdr_schema, sp_parentitem },
    { "parentOrganization", mdr_schema, sp_parentorganisation },
    { "parentService", mdr_schema, sp_parentservice },
    { "partOfEpisode", mdr_schema, sp_partofepisode },
    { "partOfInvoice", mdr_schema, sp_partofinvoice },
    { "partOfSeason", mdr_schema, sp_partofseason },
    { "partOfSeries", mdr_schema, sp_partofseries },
    { "partOfSystem", mdr_schema, sp_partofsystem },
    { "partOfTrip", mdr_schema, sp_partoftrip },
    { "participant", mdr_schema, sp_participant },
    { "partofOrder", mdr_schema, sp_partoforder },
    { "partySize", mdr_schema, sp_partysize },
    { "passengerPriorityStatus", mdr_schema, sp_passengerprioritystatus },
    { "passengerSequenceNumber", mdr_schema, sp_passengersequencenumber },
    { "pathophysiology", mdr_schema, sp_pathophysiology },
    { "pattern", mdr_schema, sp_pattern },
    { "payload", mdr_schema, sp_payload },
    { "paymentAccepted", mdr_schema, sp_paymentaccepted },
    { "paymentDue", mdr_schema, sp_paymentdue },
    { "paymentDueDate", mdr_schema, sp_paymentduedate },
    { "paymentMethod", mdr_schema, sp_paymentmethod },
    { "paymentMethodId", mdr_schema, sp_paymentmethodid },
    { "paymentStatus", mdr_schema, sp_paymentstatus },
    { "paymentUrl", mdr_schema, sp_paymenturl },
    { "penciler", mdr_schema, sp_penciler },
    { "percentile10", mdr_schema, sp_percentile10 },
    { "percentile25", mdr_schema, sp_percentile25 },
    { "percentile75", mdr_schema, sp_percentile75 },
    { "percentile90", mdr_schema, sp_percentile90 },
    { "performTime", mdr_schema, sp_performtime },
    { "performer", mdr_schema, sp_performer },
    { "performerIn", mdr_schema, sp_performerin },
    { "permissionType", mdr_schema, sp_permissiontype },
    { "permissions", mdr_schema, sp_permissions },
    { "permitAudience", mdr_schema, sp_permitaudience },
    { "permittedUsage", mdr_schema, sp_permittedusage },
    { "petsAllowed", mdr_schema, sp_petsallowed },
    { "phase", mdr_schema, sp_phase },
    { "phoneticText", mdr_schema, sp_phonetictext },
    { "photo", mdr_schema, sp_photo },
    { "physicalRequirement", mdr_schema, sp_physicalrequirement },
    { "physiologicalBenefits", mdr_schema, sp_physiologicalbenefits },
    { "pickupLocation", mdr_schema, sp_pickuplocation },
    { "pickupTime", mdr_schema, sp_pickuptime },
    { "playMode", mdr_schema, sp_playmode },
    { "playerType", mdr_schema, sp_playertype },
    { "playersOnline", mdr_schema, sp_playersonline },
    { "polygon", mdr_schema, sp_polygon },
    { "population", mdr_schema, sp_population },
    { "populationType", mdr_schema, sp_populationtype },
    { "position", mdr_schema, sp_position },
    { "possibleComplication", mdr_schema, sp_possiblecomplication },
    { "possibleTreatment", mdr_schema, sp_possibletreatment },
    { "postOfficeBoxNumber", mdr_schema, sp_postofficeboxnumber },
    { "postOp", mdr_schema, sp_postop },
    { "postalCode", mdr_schema, sp_postalcode },
    { "postalCodeBegin", mdr_schema, sp_postalcodebegin },
    { "postalCodeEnd", mdr_schema, sp_postalcodeend },
    { "postalCodePrefix", mdr_schema, sp_postalcodeprefix },
    { "postalCodeRange", mdr_schema, sp_postalcoderange },
    { "potentialAction", mdr_schema, sp_potentialaction },
    { "preOp", mdr_schema, sp_preop },
    { "predecessorOf", mdr_schema, sp_predecessorof },
    { "pregnancyCategory", mdr_schema, sp_pregnancycategory },
    { "pregnancyWarning", mdr_schema, sp_pregnancywarning },
    { "prepTime", mdr_schema, sp_preptime },
    { "preparation", mdr_schema, sp_preparation },
    { "prescribingInfo", mdr_schema, sp_prescribinginfo },
    { "prescriptionStatus", mdr_schema, sp_prescriptionstatus },
    { "previousItem", mdr_schema, sp_previousitem },
    { "previousStartDate", mdr_schema, sp_previousstartdate },
    { "price", mdr_schema, sp_price },
    { "priceComponent", mdr_schema, sp_pricecomponent },
    { "priceComponentType", mdr_schema, sp_pricecomponenttype },
    { "priceCurrency", mdr_schema, sp_pricecurrency },
    { "priceRange", mdr_schema, sp_pricerange },
    { "priceSpecification", mdr_schema, sp_pricespecification },
    { "priceType", mdr_schema, sp_pricetype },
    { "priceValidUntil", mdr_schema, sp_pricevaliduntil },
    { "primaryImageOfPage", mdr_schema, sp_primaryimageofpage },
    { "primaryPrevention", mdr_schema, sp_primaryprevention },
    { "printColumn", mdr_schema, sp_printcolumn },
    { "printEdition", mdr_schema, sp_printedition },
    { "printPage", mdr_schema, sp_printpage },
    { "printSection", mdr_schema, sp_printsection },
    { "procedure", mdr_schema, sp_procedure },
    { "procedureType", mdr_schema, sp_proceduretype },
    { "processingTime", mdr_schema, sp_processingtime },
    { "processorRequirements", mdr_schema, sp_processorrequirements },
    { "producer", mdr_schema, sp_producer },
    { "productGroupID", mdr_schema, sp_productgroupid },
    { "productID", mdr_schema, sp_productid },
    { "productSupported", mdr_schema, sp_productsupported },
    { "productionCompany", mdr_schema, sp_productioncompany },
    { "productionDate", mdr_schema, sp_productiondate },
    { "proficiencyLevel", mdr_schema, sp_proficiencylevel },
    { "programMembershipUse", mdr_schema, sp_programmemembershipused },
    { "programName", mdr_schema, sp_programmename },
    { "programPrequisites", mdr_schema, sp_programprequisites },
    { "programType", mdr_schema, sp_programtype },
    { "programmingLanguage", mdr_schema, sp_programminglanguage },
    { "programmingModel", mdr_schema, sp_programmingmodel },
    { "propertyID", mdr_schema, sp_propertyid },
    { "propriatryName", mdr_schema, sp_propriatryname },
    { "proprietaryName", mdr_schema, sp_proprietaryname },
    { "proteinContent", mdr_schema, sp_proteincontent },
    { "provider", mdr_schema, sp_provider },
    { "providerMobility", mdr_schema, sp_providermobility },
    { "providesBroadcastService", mdr_schema, sp_providesbroadcastservice },
    { "providesService", mdr_schema, sp_providesservice },
    { "publicAccess", mdr_schema, sp_publicaccess },
    { "publicTransportClosuresInfo", mdr_schema, sp_publictransportclosuresinfo },
    { "publication", mdr_schema, sp_publication },
    { "publicationType", mdr_schema, sp_publicationtype },
    { "publishedBy", mdr_schema, sp_publishedby },
    { "publishedOn", mdr_schema, sp_publishedon },
    { "publisher", mdr_schema, sp_publisher },
    { "publisherImprint", mdr_schema, sp_publisherimprint },
    { "publishingPrinciples", mdr_schema, sp_publishingprinciples },
    { "purchaseDate", mdr_schema, sp_purchasedate },
    { "purpose", mdr_schema, sp_purpose },
    { "qualifications", mdr_schema, sp_qualifications },
    { "quarantineGuidelines", mdr_schema, sp_quarantineguidelines },
    { "query", mdr_schema, sp_query },
    { "quest", mdr_schema, sp_quest },
    { "question", mdr_schema, sp_question },
    { "rangeIncludes", mdr_schema, sp_rangeincludes },
    { "ratingCount", mdr_schema, sp_ratingcount },
    { "ratingExplanation", mdr_schema, sp_ratingexplanation },
    { "ratingValue", mdr_schema, sp_ratingvalue },
    { "readBy", mdr_schema, sp_readby },
    { "readonlyValue", mdr_schema, sp_readonlyvalue },
    { "realEstateAgent", mdr_schema, sp_realestateagent },
    { "recipe", mdr_schema, sp_recipe },
    { "recipeCategory", mdr_schema, sp_recipecategory },
    { "recipeCuisine", mdr_schema, sp_recipecuisine },
    { "recipeIngredient", mdr_schema, sp_recipeingredient },
    { "recipeInstructions", mdr_schema, sp_recipeinstructions },
    { "recipeYield", mdr_schema, sp_recipeyield },
    { "recipient", mdr_schema, sp_recipient },
    { "recognisedBy", mdr_schema, sp_recognisedby },
    { "recognisingAuthority", mdr_schema, sp_recognisingauthority },
    { "recommendationStrength", mdr_schema, sp_recommendationstrength },
    { "recommendedIntake", mdr_schema, sp_recommendedintake },
    { "recordLabel", mdr_schema, sp_recordlabel },
    { "recordedAs", mdr_schema, sp_recordedas },
    { "recordedAt", mdr_schema, sp_recordedat },
    { "recordedIn", mdr_schema, sp_recordedin },
    { "recordingOf", mdr_schema, sp_recordingof },
    { "recourseLoan", mdr_schema, sp_recourseloan },
    { "referenceQuantity", mdr_schema, sp_referencequantity },
    { "referencesOrder", mdr_schema, sp_referencesorder },
    { "refundType", mdr_schema, sp_refundtype },
    { "regionDrained", mdr_schema, sp_regiondrained },
    { "regionsAllowed", mdr_schema, sp_regionsallowed },
    { "relatedAnatomy", mdr_schema, sp_relatedanatomy },
    { "relatedCondition", mdr_schema, sp_relatedcondition },
    { "relatedDrug", mdr_schema, sp_relateddrug },
    { "relatedLink", mdr_schema, sp_relatedlink },
    { "relatedStructure", mdr_schema, sp_relatedstructure },
    { "relatedTherapy", mdr_schema, sp_relatedtherapy },
    { "relatedTo", mdr_schema, sp_relatedto },
    { "releaseDate", mdr_schema, sp_releasedate },
    { "releaseNotes", mdr_schema, sp_releasenotes },
    { "releaseOf", mdr_schema, sp_releaseof },
    { "releasedEvent", mdr_schema, sp_releasedevent },
    { "relevantOccupation", mdr_schema, sp_relevantoccupation },
    { "relevantSpeciality", mdr_schema, sp_relevantspeciality },
    { "remainingAttendeeCapacity", mdr_schema, sp_remainingattendeecapacity },
    { "renegotiableLoan", mdr_schema, sp_renegotiableloan },
    { "repeatCount", mdr_schema, sp_repeatcount },
    { "repeatFrequency", mdr_schema, sp_repeatfrequency },
    { "repetitions", mdr_schema, sp_repetitions },
    { "replacee", mdr_schema, sp_replacee },
    { "replacer", mdr_schema, sp_replacer },
    { "replyToUrl", mdr_schema, sp_replytourl },
    { "reportNumber", mdr_schema, sp_reportnumber },
    { "representativeOfPage", mdr_schema, sp_representativeofpage },
    { "requiredCollateral", mdr_schema, sp_requiredcollateral },
    { "requiredGender", mdr_schema, sp_requiredgender },
    { "requiredMaxAge", mdr_schema, sp_requiredmaxage },
    { "requiredMinAge", mdr_schema, sp_requiredminage },
    { "requiredQuantity", mdr_schema, sp_requiredquantity },
    { "requiresSubscription", mdr_schema, sp_requiressubscription },
    { "reservationFor", mdr_schema, sp_reservationfor },
    { "reservationId", mdr_schema, sp_reservationid },
    { "reservationStatus", mdr_schema, sp_reservationstatus },
    { "reservedTicket", mdr_schema, sp_reservedticket },
    { "responsibilities", mdr_schema, sp_responsibilities },
    { "restPeriods", mdr_schema, sp_restperiods },
    { "result", mdr_schema, sp_result },
    { "resultComment", mdr_schema, sp_resultcomment },
    { "resultReview", mdr_schema, sp_resultreview },
    { "returnFees", mdr_schema, sp_returnfees },
    { "returnPolicyCategory", mdr_schema, sp_returnpolicycategory },
    { "review", mdr_schema, sp_review },
    { "reviewAspect", mdr_schema, sp_reviewaspect },
    { "reviewBody", mdr_schema, sp_reviewbody },
    { "reviewCount", mdr_schema, sp_reviewcount },
    { "reviewRating", mdr_schema, sp_reviewrating },
    { "reviewedBy", mdr_schema, sp_reviewedby },
    { "riskFactor", mdr_schema, sp_riskfactor },
    { "risks", mdr_schema, sp_risks },
    { "roleName", mdr_schema, sp_rolename },
    { "roofLoad", mdr_schema, sp_roofload },
    { "rsvpResponse", mdr_schema, sp_rsvpresponse },
    { "runsTo", mdr_schema, sp_runsto },
    { "runtimePlatform", mdr_schema, sp_runtimeplatform },
    { "rxcui", mdr_schema, sp_rxcui },
    { "safetyConsideration", mdr_schema, sp_safetyconsideration },
    { "salaryCurrency", mdr_schema, sp_salarycurrency },
    { "salaryUponCompletion", mdr_schema, sp_salaryuponcompletion },
    { "sameAs", mdr_schema, sp_sameas },
    { "saturatedFatContent", mdr_schema, sp_saturatedfatcontent },
    { "scheduledPaymentDate", mdr_schema, sp_scheduledpaymentdate },
    { "scheduledTime", mdr_schema, sp_scheduledtime },
    { "schemaVersion", mdr_schema, sp_schemaversion },
    { "schoolClosuresInfo", mdr_schema, sp_schoolclosuresinfo },
    { "screenCount", mdr_schema, sp_screencount },
    { "screenshot", mdr_schema, sp_screenshot },
    { "sdDatePublished", mdr_schema, sp_sddatepublished },
    { "sdLicense", mdr_schema, sp_sdlicense },
    { "sdPublisher", mdr_schema, sp_sdpublisher },
    { "seasonNumber", mdr_schema, sp_seasonnumber },
    { "seatNumber", mdr_schema, sp_seatnumber },
    { "seatRow", mdr_schema, sp_seatrow },
    { "seatSection", mdr_schema, sp_seatsection },
    { "seatingCapacity", mdr_schema, sp_seatingcapacity },
    { "seatingType", mdr_schema, sp_seatingtype },
    { "secondaryPrevention", mdr_schema, sp_secondaryprevention },
    { "securityClearanceRequirement", mdr_schema, sp_securityclearancerequirement },
    { "securityScreening", mdr_schema, sp_securityscreening },
    { "seeks", mdr_schema, sp_seeks },
    { "seller", mdr_schema, sp_seller },
    { "sender", mdr_schema, sp_sender },
    { "sensoryClearanceRequirement", mdr_schema, sp_sensoryclearancerequirement },
    { "sensoryUnit", mdr_schema, sp_sensoryunit },
    { "serialNumber", mdr_schema, sp_serialnumber },
    { "seriousAdverseOutcome", mdr_schema, sp_seriousadverseoutcome },
    { "serverStatus", mdr_schema, sp_serverstatus },
    { "servesCuisine", mdr_schema, sp_servescuisine },
    { "serviceArea", mdr_schema, sp_servicearea },
    { "serviceLocation", mdr_schema, sp_servicelocation },
    { "serviceOperator", mdr_schema, sp_serviceoperator },
    { "serviceOutput", mdr_schema, sp_serviceoutput },
    { "servicePhone", mdr_schema, sp_servicephone },
    { "servicePostalAddress", mdr_schema, sp_servicepostaladdress },
    { "serviceSmsNumber", mdr_schema, sp_servicesmsnumber },
    { "serviceType", mdr_schema, sp_servicetype },
    { "serviceUrl", mdr_schema, sp_serviceurl },
    { "servingSize", mdr_schema, sp_servingsize },
    { "sharedContent", mdr_schema, sp_sharedcontent },
    { "shippingDestination", mdr_schema, sp_shippingdestination },
    { "shippingLabel", mdr_schema, sp_shippinglabel },
    { "shippingRate", mdr_schema, sp_shippingrate },
    { "shippingSettingsLink", mdr_schema, sp_shippingsettingslink },
    { "sibling", mdr_schema, sp_sibling },
    { "signDetected", mdr_schema, sp_signdetected },
    { "signOrSymptom", mdr_schema, sp_signorsymptom },
    { "significance", mdr_schema, sp_significance },
    { "significantLink", mdr_schema, sp_significantlink },
    { "size", mdr_schema, sp_size },
    { "sizeGroup", mdr_schema, sp_sizegroup },
    { "sizeSystem", mdr_schema, sp_sizesystem },
    { "skills", mdr_schema, sp_skills },
    { "sku", mdr_schema, sp_sku },
    { "slogan", mdr_schema, sp_slogan },
    { "smokingAllowed", mdr_schema, sp_smokingallowed },
    { "sodiumContent", mdr_schema, sp_sodiumcontent },
    { "softwareAddOn", mdr_schema, sp_softwareaddon },
    { "softwareHelp", mdr_schema, sp_softwarehelp },
    { "softwareRequirements", mdr_schema, sp_softwarerequirements },
    { "softwareVersion", mdr_schema, sp_softwareversion },
    { "source", mdr_schema, sp_source },
    { "sourceOrganization", mdr_schema, sp_sourceorganisation },
    { "sourcedFrom", mdr_schema, sp_sourcedfrom },
    { "spatial", mdr_schema, sp_spatial },
    { "spatialCoverage", mdr_schema, sp_spatialcoverage },
    { "speakable", mdr_schema, sp_speakable },
    { "specialCommitments", mdr_schema, sp_specialcommitments },
    { "specialOpeningHoursSpecification", mdr_schema, sp_specialopeninghoursspecification },
    { "specialty", mdr_schema, sp_speciality },
    { "speechToTextMarkup", mdr_schema, sp_speechtotextmarkup },
    { "speed", mdr_schema, sp_speed },
    { "spokenByCharacter", mdr_schema, sp_spokenbycharacter },
    { "sponsor", mdr_schema, sp_sponsor },
    { "sport", mdr_schema, sp_sport },
    { "sportsActivityLocation", mdr_schema, sp_sportsactivitylocation },
    { "sportsEvent", mdr_schema, sp_sportsevent },
    { "sportsTeam", mdr_schema, sp_sportsteam },
    { "spouse", mdr_schema, sp_spouse },
    { "sscheduleTimezone", mdr_schema, sp_scheduletimezone },
    { "stage", mdr_schema, sp_stage },
    { "stageAsNumber", mdr_schema, sp_stageasnumber },
    { "starRating", mdr_schema, sp_starrating },
    { "startDate", mdr_schema, sp_startdate },
    { "startOffset", mdr_schema, sp_startoffset },
    { "startTime", mdr_schema, sp_starttime },
    { "status", mdr_schema, sp_status },
    { "steeringPosition", mdr_schema, sp_steeringposition },
    { "step", mdr_schema, sp_step },
    { "stepValue", mdr_schema, sp_stepvalue },
    { "steps", mdr_schema, sp_steps },
    { "storageRequirements", mdr_schema, sp_storagerequirements },
    { "streetAddress", mdr_schema, sp_streetaddress },
    { "strengthUnit", mdr_schema, sp_strengthunit },
    { "strengthValue", mdr_schema, sp_strengthvalue },
    { "structuralClass", mdr_schema, sp_structuralclass },
    { "study", mdr_schema, sp_study },
    { "studyDesign", mdr_schema, sp_studydesign },
    { "studyLocation", mdr_schema, sp_studylocation },
    { "studySubject", mdr_schema, sp_studysubject },
    { "stupidProperty", mdr_schema, sp_stupidproperty },
    { "subEvent", mdr_schema, sp_subevent },
    { "subOrganization", mdr_schema, sp_suborganisation },
    { "subReservation", mdr_schema, sp_subreservation },
    { "subStageSuffix", mdr_schema, sp_substagesuffix },
    { "subStructure", mdr_schema, sp_substructure },
    { "subTrip", mdr_schema, sp_subtrip },
    { "subjectOf", mdr_schema, sp_subjectof },
    { "subtitleLanguage", mdr_schema, sp_subtitlelanguage },
    { "subtype", mdr_schema, sp_subtype },
    { "successorOf", mdr_schema, sp_successorof },
    { "sugarContent", mdr_schema, sp_sugarcontent },
    { "suggestedAge", mdr_schema, sp_suggestedage },
    { "suggestedAnswer", mdr_schema, sp_suggestedanswer },
    { "suggestedGender", mdr_schema, sp_suggestedgender },
    { "suggestedMaxAge", mdr_schema, sp_suggestedmaxage },
    { "suggestedMeasurement", mdr_schema, sp_suggestedmeasurement },
    { "suggestedMinAge", mdr_schema, sp_suggestedminage },
    { "suitableForDiet", mdr_schema, sp_suitablefordiet },
    { "superEvent", mdr_schema, sp_superevent },
    { "supersededBy", mdr_schema, sp_supersededby },
    { "supply", mdr_schema, sp_supply },
    { "supplyTo", mdr_schema, sp_supplyto },
    { "supportingData", mdr_schema, sp_supportingdata },
    { "suspersededBy", mdr_schema, sp_suspersededby },
    { "target", mdr_schema, sp_target },
    { "targetCollection", mdr_schema, sp_targetcollection },
    { "targetDescription", mdr_schema, sp_targetdescription },
    { "targetName", mdr_schema, sp_targetname },
    { "targetPlatform", mdr_schema, sp_targetplatform },
    { "targetPopulation", mdr_schema, sp_targetpopulation },
    { "targetUrl", mdr_schema, sp_targeturl },
    { "taxID", mdr_schema, sp_taxid },
    { "teaches", mdr_schema, sp_teaches },
    { "telephone", mdr_schema, sp_telephone },
    { "temporal", mdr_schema, sp_temporal },
    { "temporalCoverage", mdr_schema, sp_temporalcoverage },
    { "termCode", mdr_schema, sp_termcode },
    { "termDuration", mdr_schema, sp_termduration },
    { "termsOfService", mdr_schema, sp_termsofservice },
    { "termsPerYear", mdr_schema, sp_termsperyear },
    { "text", mdr_schema, sp_text },
    { "textValue", mdr_schema, sp_textvalue },
    { "thumbnail", mdr_schema, sp_thumbnail },
    { "thumbnailUrl", mdr_schema, sp_thumbnailurl },
    { "tickerSymbol", mdr_schema, sp_tickersymbol },
    { "ticketNumber", mdr_schema, sp_ticketnumber },
    { "ticketToken", mdr_schema, sp_tickettoken },
    { "ticketedSeat", mdr_schema, sp_ticketedseat },
    { "timeOfDay", mdr_schema, sp_timeofday },
    { "timeRequired", mdr_schema, sp_timerequired },
    { "timeToComplete", mdr_schema, sp_timetocomplete },
    { "tissueSample", mdr_schema, sp_tissuesample },
    { "title", mdr_schema, sp_title },
    { "titleEIDR", mdr_schema, sp_titleeidr },
    { "toLocation", mdr_schema, sp_tolocation },
    { "toRecipient", mdr_schema, sp_torecipient },
    { "tocContinuation", mdr_schema, sp_toccontinuation },
    { "tocEntry", mdr_schema, sp_tocentry },
    { "tongueWeight", mdr_schema, sp_tongueweight },
    { "tool", mdr_schema, sp_tool },
    { "torque", mdr_schema, sp_torque },
    { "totalJobOpenings", mdr_schema, sp_totaljobopenings },
    { "totalPaymentDue", mdr_schema, sp_totalpaymentdue },
    { "totalPrice", mdr_schema, sp_totalprice },
    { "totalTime", mdr_schema, sp_totaltime },
    { "tourBookingPage", mdr_schema, sp_tourbookingpage },
    { "touristType", mdr_schema, sp_touristtype },
    { "track", mdr_schema, sp_track },
    { "trackingNumber", mdr_schema, sp_trackingnumber },
    { "trackingUrl", mdr_schema, sp_trackingurl },
    { "trailer", mdr_schema, sp_trailer },
    { "trailerWeight", mdr_schema, sp_trailerweight },
    { "trainName", mdr_schema, sp_trainname },
    { "trainNumber", mdr_schema, sp_trainnumber },
    { "trainingSalary", mdr_schema, sp_trainingsalary },
    { "transFatContent", mdr_schema, sp_transfatcontent },
    { "transcript", mdr_schema, sp_transcript },
    { "transitTime", mdr_schema, sp_transittime },
    { "transitTimeLabel", mdr_schema, sp_transittimelabel },
    { "translationOfWork", mdr_schema, sp_translationofwork },
    { "translator", mdr_schema, sp_translator },
    { "transmissionMethod", mdr_schema, sp_transmissionmethod },
    { "travelBans", mdr_schema, sp_travelbans },
    { "trialDesign", mdr_schema, sp_trialdesign },
    { "tributary", mdr_schema, sp_tributary },
    { "typeOfBed", mdr_schema, sp_typeofbed },
    { "typeOfGood", mdr_schema, sp_typeofgood },
    { "typicalAgeRange", mdr_schema, sp_typicalagerange },
    { "typicalCreditsPerTerm", mdr_schema, sp_typicalcreditsperterm },
    { "typicalTest", mdr_schema, sp_typicaltest },
    { "underName", mdr_schema, sp_undername },
    { "unitCode", mdr_schema, sp_unitcode },
    { "unitText", mdr_schema, sp_unittext },
    { "unnamedSourcesPolicy", mdr_schema, sp_unnamedsourcespolicy },
    { "unsaturatedFatContent", mdr_schema, sp_unsaturatedfatcontent },
    { "uploadDate", mdr_schema, sp_uploaddate },
    { "upvoteCount", mdr_schema, sp_upvotecount },
    { "url", mdr_schema, sp_url },
    { "urlTemplate", mdr_schema, sp_urltemplate },
    { "usageInfo", mdr_schema, sp_usageinfo },
    { "usedToDiagnose", mdr_schema, sp_usedtodiagnose },
    { "userInteractionCount", mdr_schema, sp_userinteractioncount },
    { "usesDevice", mdr_schema, sp_usesdevice },
    { "usesHealthPlanIdStandard", mdr_schema, sp_useshealthplanidstandard },
    { "utterances", mdr_schema, sp_utterances },
    { "validFor", mdr_schema, sp_validfor },
    { "validFrom", mdr_schema, sp_validfrom },
    { "validIn", mdr_schema, sp_validin },
    { "validThrough", mdr_schema, sp_validthrough },
    { "validUntil", mdr_schema, sp_validuntil },
    { "value", mdr_schema, sp_value },
    { "valueAddedTaxIncluded", mdr_schema, sp_valueaddedtaxincluded },
    { "valueMaxLength", mdr_schema, sp_valuemaxlength },
    { "valueMinLength", mdr_schema, sp_valueminlength },
    { "valueName", mdr_schema, sp_valuename },
    { "valuePattern", mdr_schema, sp_valuepattern },
    { "valueReference", mdr_schema, sp_valuereference },
    { "valueRequired", mdr_schema, sp_valuerequired },
    { "variableMeasured", mdr_schema, sp_variablemeasured },
    { "variablesMeasured", mdr_schema, sp_variablesmeasured },
    { "variantCover", mdr_schema, sp_variantcover },
    { "variesBy", mdr_schema, sp_variesby },
    { "vatID", mdr_schema, sp_vatid },
    { "vehicleConfiguration", mdr_schema, sp_vehicleconfiguration },
    { "vehicleEngine", mdr_schema, sp_vehicleengine },
    { "vehicleIdentificationNumber", mdr_schema, sp_vehicleidentificationnumber },
    { "vehicleInteriorColor", mdr_schema, sp_vehicleinteriorcolour },
    { "vehicleInteriorType", mdr_schema, sp_vehicleinteriortype },
    { "vehicleModelDate", mdr_schema, sp_vehiclemodeldate },
    { "vehicleSeatingCapacity", mdr_schema, sp_vehicleseatingcapacity },
    { "vehicleSpecialUsage", mdr_schema, sp_vehiclespecialusage },
    { "vehicleTransmission", mdr_schema, sp_vehicletransmission },
    { "verificationFactCheckingPolicy", mdr_schema, sp_verificationfactcheckingpolicy },
    { "version", mdr_schema, sp_version },
    { "video", mdr_schema, sp_video },
    { "videoFormat", mdr_schema, sp_videoformat },
    { "videoFrameSize", mdr_schema, sp_videoframesize },
    { "videoQuality", mdr_schema, sp_videoquality },
    { "volumeNumber", mdr_schema, sp_volumenumber },
    { "warning", mdr_schema, sp_warning },
    { "warranty", mdr_schema, sp_warranty },
    { "warrantyScope", mdr_schema, sp_warrantyscope },
    { "webCheckInTime", mdr_schema, sp_webcheckintime },
    { "webFeed", mdr_schema, sp_webfeed },
    { "weight", mdr_schema, sp_weight },
    { "weightTotal", mdr_schema, sp_weighttotal },
    { "wheelbase", mdr_schema, sp_wheelbase },
    { "width", mdr_schema, sp_width },
    { "winner", mdr_schema, sp_winner },
    { "wordCount", mdr_schema, sp_wordcount },
    { "workExample", mdr_schema, sp_workexample },
    { "workFeatured", mdr_schema, sp_workfeatured },
    { "workHours", mdr_schema, sp_workhours },
    { "workLocation", mdr_schema, sp_worklocation },
    { "workPerformed", mdr_schema, sp_workperformed },
    { "workPresented", mdr_schema, sp_workpresented },
    { "workTranslation", mdr_schema, sp_worktranslation },
    { "workload", mdr_schema, sp_workload },
    { "worksFor", mdr_schema, sp_worksfor },
    { "worstRating", mdr_schema, sp_worstrating },
    { "xpath", mdr_schema, sp_xpath },
    { "yearBuilt", mdr_schema, sp_yearbuilt },
    { "yearlyRevenue", mdr_schema, sp_yearlyrevenue },
    { "yearsInOperation", mdr_schema, sp_yearsinoperation },
    { "yield", mdr_schema, sp_yield },

    // living standard / microformats v1
    { "additional-name", mdr_microformats, mp_additional_name },
    { "adr", mdr_microformats, mp_adr },
    { "affliation", mdr_microformats, mp_affliation },
    { "agent", mdr_microformats, mp_agent },
    { "album", mdr_microformats, mp_album },
    { "anniversary", mdr_microformats, mp_anniversary },
    { "attach", mdr_microformats, mp_attach },
    { "attendee", mdr_microformats, mp_attendee },
    { "author", mdr_microformats, mp_author },
    { "availability", mdr_microformats, mp_availability },
    { "bday", mdr_microformats, mp_bday },
    { "best", mdr_microformats, mp_best },
    { "bookmark", mdr_microformats, mp_bookmark },
    { "categories", mdr_microformats, mp_categories },
    { "category", mdr_microformats, mp_category },
    { "class", mdr_microformats, mp_class },
    { "comment", mdr_microformats, mp_comment },
    { "contact", mdr_microformats, mp_contact },
    { "contributor", mdr_microformats, mp_contributor },
    { "count", mdr_microformats, mp_count },
    { "country-name", mdr_microformats, mp_country_name },
    { "created", mdr_microformats, mp_created },
    { "dateline", mdr_microformats, mp_dateline },
    { "description", mdr_microformats, mp_description },
    { "dtend", mdr_microformats, mp_dtend },
    { "dtexpired", mdr_microformats, mp_dtexpired },
    { "dtlisted", mdr_microformats, mp_dtlisted },
    { "dtreviewed", mdr_microformats, mp_dtreviewed },
    { "dtstart", mdr_microformats, mp_dtstart },
    { "duration", mdr_microformats, mp_duration },
    { "education", mdr_microformats, mp_education },
    { "email", mdr_microformats, mp_email },
    { "enclosure", mdr_microformats, mp_enclosure },
    { "entry", mdr_microformats, mp_entry },
    { "entry-content", mdr_microformats, mp_entry_content },
    { "entry-summary", mdr_microformats, mp_entry_summary },
    { "entry-title", mdr_microformats, mp_entry_title },
    { "exdate", mdr_microformats, mp_exdate },
    { "experience", mdr_microformats, mp_experience },
    { "extended-address", mdr_microformats, mp_extended_address },
    { "family-name", mdr_microformats, mp_family_name },
    { "feed-category", mdr_microformats, mp_feed_category },
    { "fn", mdr_microformats, mp_fn },
    { "gender-identity", mdr_microformats, mp_gender_identity },
    { "geo", mdr_microformats, mp_geo },
    { "given-name", mdr_microformats, mp_given_name },
    { "h", mdr_microformats, mp_h },
    { "hentry", mdr_microformats, mp_hentry },
    { "honourific-prefix", mdr_microformats, mp_honourific_prefix },
    { "honourific-suffixr", mdr_microformats, mp_honourific_suffix },
    { "impp", mdr_microformats, mp_impp },
    { "include", mdr_microformats, mp_include },
    { "info", mdr_microformats, mp_info },
    { "ingredient", mdr_microformats, mp_ingredient },
    { "instructions", mdr_microformats, mp_instructions },
    { "item", mdr_microformats, mp_item },
    { "item-license", mdr_microformats, mp_licence },
    { "key", mdr_microformats, mp_key },
    { "kind", mdr_microformats, mp_kind },
    { "label", mdr_microformats, mp_label },
    { "lang", mdr_microformats, mp_lang },
    { "last-modified", mdr_microformats, mp_last_modified },
    { "latitude", mdr_microformats, mp_latitude },
    { "license", mdr_microformats, mp_licence },
    { "lister", mdr_microformats, mp_lister },
    { "listing-action", mdr_microformats, mp_listing_action },
    { "locality", mdr_microformats, mp_locality },
    { "location", mdr_microformats, mp_location },
    { "logo", mdr_microformats, mp_logo },
    { "longitude", mdr_microformats, mp_longitude },
    { "mailer", mdr_microformats, mp_mailer },
    { "member", mdr_microformats, mp_member },
    { "method", mdr_microformats, mp_method },
    { "min", mdr_microformats, mp_min },
    { "n", mdr_microformats, mp_n },
    { "nickname", mdr_microformats, mp_nickname },
    { "note", mdr_microformats, mp_note },
    { "nutrition", mdr_microformats, mp_nutrition },
    { "org", mdr_microformats, mp_org },
    { "organisation-name", mdr_microformats, mp_organisation_name },
    { "organisation-unit", mdr_microformats, mp_organisation_unit },
    { "organizer", mdr_microformats, mp_organiser },
    { "partstat", mdr_microformats, mp_partstat },
    { "payment", mdr_microformats, mp_payment },
    { "permalink", mdr_microformats, mp_permalink },
    { "photo", mdr_microformats, mp_photo },
    { "player", mdr_microformats, mp_player },
    { "position", mdr_microformats, mp_position },
    { "post-office-box", mdr_microformats, mp_post_office_box },
    { "postal-code", mdr_microformats, mp_postal_code },
    { "price", mdr_microformats, mp_price },
    { "principles", mdr_microformats, mp_principles },
    { "publications", mdr_microformats, mp_publications },
    { "published", mdr_microformats, mp_published },
    { "rating", mdr_microformats, mp_rating },
    { "rdate", mdr_microformats, mp_rdate },
    { "region", mdr_microformats, mp_region },
    { "rel", mdr_microformats, mp_rel },
    { "related", mdr_microformats, mp_related },
    { "rev", mdr_microformats, mp_rev },
    { "review", mdr_microformats, mp_review },
    { "reviewer", mdr_microformats, mp_reviewer },
    { "role", mdr_microformats, mp_role },
    { "rrule", mdr_microformats, mp_rrule },
    { "resources", mdr_microformats, mp_resources },
    { "s", mdr_microformats, mp_s },
    { "sample", mdr_microformats, mp_sample },
    { "sequence", mdr_microformats, mp_sequence },
    { "self", mdr_microformats, mp_self },
    { "sex", mdr_microformats, mp_sex },
    { "skill", mdr_microformats, mp_skill },
    { "skills", mdr_microformats, mp_skill },
    { "sort-string", mdr_microformats, mp_sort_string },
    { "sound", mdr_microformats, mp_sound },
    { "source-org", mdr_microformats, mp_source_org },
    { "status", mdr_microformats, mp_status },
    { "street-address", mdr_microformats, mp_street_address },
    { "summary", mdr_microformats, mp_summary },
    { "tag", mdr_microformats, mp_tag },
    { "tags", mdr_microformats, mp_tags },
    { "tel", mdr_microformats, mp_tel },
    { "term", mdr_microformats, mp_term },
    { "title", mdr_microformats, mp_title },
    { "transp", mdr_microformats, mp_transp },
    { "type", mdr_microformats, mp_type },
    { "tz", mdr_microformats, mp_tz },
    { "uid", mdr_microformats, mp_uid },
    { "updated", mdr_microformats, mp_updated },
    { "url", mdr_microformats, mp_url },
    { "value", mdr_microformats, mp_value },
    { "value-title", mdr_microformats, mp_value_title },
    { "version", mdr_microformats, mp_version },
    { "votes", mdr_microformats, mp_votes },
    { "worst", mdr_microformats, mp_worst },
    { "yield", mdr_microformats, mp_yield },

    // living standard
    { "author", mdr_whatwg, wp_author },
    { "license", mdr_whatwg, wp_license },
    { "title", mdr_whatwg, wp_title },
    { "work", mdr_whatwg, wp_work },

    // microformats v2
    { "dt-accessed", mdr_microformats, mp2_dt_accessed },
    { "dt-anniversary", mdr_microformats, mp2_dt_anniversary },
    { "dt-bday", mdr_microformats, mp2_dt_bday },
    { "dt-duration", mdr_microformats, mp2_dt_duration },
    { "dt-end", mdr_microformats, mp2_dt_end },
    { "dt-expired", mdr_microformats, mp2_dt_expired },
    { "dt-listed", mdr_microformats, mp2_dt_listed },
    { "dt-published", mdr_microformats, mp2_dt_published },
    { "dt-start", mdr_microformats, mp2_dt_start },
    { "dt-updated", mdr_microformats, mp2_dt_updated },
    { "e-content", mdr_microformats, mp2_e_content },
    { "e-description", mdr_microformats, mp2_e_description },
    { "e-instructions", mdr_microformats, mp2_e_instructions },
    { "h-breadcrumb", mdr_microformats, mp2_h_breadcrumb },
    { "h-entry", mdr_microformats, mp2_h_entry },
    { "p-action", mdr_microformats, mp2_p_action },
    { "p-additional-name", mdr_microformats, mp2_p_additional_name },
    { "p-adr", mdr_microformats, mp2_p_adr },
    { "p-affiliation", mdr_microformats, mp2_p_affiliation },
    { "p-altitude", mdr_microformats, mp2_p_altitude },
    { "p-attendee", mdr_microformats, mp2_p_attendee },
    { "p-author", mdr_microformats, mp2_p_author },
    { "p-best", mdr_microformats, mp2_p_best },
    { "p-category", mdr_microformats, mp2_p_category },
    { "p-comment", mdr_microformats, mp2_p_comment },
    { "p-contact", mdr_microformats, mp2_p_contact },
    { "p-content", mdr_microformats, mp2_p_content },
    { "p-country-name", mdr_microformats, mp2_p_country_name },
    { "p-description", mdr_microformats, mp2_p_description },
    { "p-duration", mdr_microformats, mp2_p_duration },
    { "p-education", mdr_microformats, mp2_p_education },
    { "p-entry", mdr_microformats, mp2_p_entry },
    { "p-experience", mdr_microformats, mp2_p_experience },
    { "p-extended-address", mdr_microformats, mp2_p_extended_address },
    { "p-family-name", mdr_microformats, mp2_p_family_name },
    { "p-region", mdr_microformats, mp2_p_region },
    { "p-gender-identity", mdr_microformats, mp2_p_gender_identity },
    { "p-geo", mdr_microformats, mp2_p_geo },
    { "p-given_name", mdr_microformats, mp2_p_given_name },
    { "p-honourific-prefix", mdr_microformats, mp2_p_honourific_prefix },
    { "p-honourific-suffix", mdr_microformats, mp2_p_honourific_suffix },
    { "p-ingredient", mdr_microformats, mp2_p_ingredient },
    { "p-item", mdr_microformats, mp2_p_item },
    { "p-job-title", mdr_microformats, mp2_p_job_title },
    { "p-label", mdr_microformats, mp2_p_label },
    { "p-latitude", mdr_microformats, mp2_p_latitude },
    { "p-lister", mdr_microformats, mp2_p_lister },
    { "p-locality", mdr_microformats, mp2_p_locality },
    { "p-location", mdr_microformats, mp2_p_location },
    { "p-longitude", mdr_microformats, mp2_p_longitude },
    { "p-name", mdr_microformats, mp2_p_name },
    { "p-nickname", mdr_microformats, mp2_p_nickname },
    { "p-note", mdr_microformats, mp2_p_note },
    { "p-nutrition", mdr_microformats, mp2_p_nutrition },
    { "p-post-office-box", mdr_microformats, mp2_p_post_office_box },
    { "p-postal-code", mdr_microformats, mp2_p_postal_code },
    { "p-publication", mdr_microformats, mp2_p_publication },
    { "p-org", mdr_microformats, mp2_p_org },
    { "p-price", mdr_microformats, mp2_p_price },
    { "p-rating", mdr_microformats, mp2_p_rating },
    { "p-read-of", mdr_microformats, mp2_p_read_of },
    { "p-role", mdr_microformats, mp2_p_role },
    { "p-rsvp", mdr_microformats, mp2_p_rsvp },
    { "p-sex", mdr_microformats, mp2_p_sex },
    { "p-size", mdr_microformats, mp2_p_size },
    { "p-skill", mdr_microformats, mp2_p_skill },
    { "p-sort-string", mdr_microformats, mp2_p_sort_string },
    { "p-street-address", mdr_microformats, mp2_p_street_address },
    { "p-summary", mdr_microformats, mp2_p_summary },
    { "p-tz", mdr_microformats, mp2_p_tz },
    { "p-worst", mdr_microformats, mp2_p_worst },
    { "p-yield", mdr_microformats, mp2_p_yield },
    { "u-audio", mdr_microformats, mp2_u_audio },
    { "u-bookmark-of", mdr_microformats, mp2_u_bookmark_of },
    { "u-email", mdr_microformats, mp2_u_email },
    { "u-featured", mdr_microformats, mp2_u_featured },
    { "u-geo", mdr_microformats, mp2_u_geo },
    { "u-impp", mdr_microformats, mp2_u_impp },
    { "u-in-reply-to", mdr_microformats, mp2_u_in_reply_to },
    { "u-like", mdr_microformats, mp2_u_like },
    { "u-like-of", mdr_microformats, mp2_u_like_of },
    { "u-listen-of", mdr_microformats, mp2_u_listen_of },
    { "u-logo", mdr_microformats, mp2_u_logo },
    { "u-key", mdr_microformats, mp2_u_key },
    { "u-photo", mdr_microformats, mp2_u_photo },
    { "u-read-of", mdr_microformats, mp2_u_read_of },
    { "u-repost", mdr_microformats, mp2_u_repost },
    { "u-repost-of", mdr_microformats, mp2_u_repost_of },
    { "u-sound", mdr_microformats, mp2_u_sound },
    { "u-syndication", mdr_microformats, mp2_u_syndication },
    { "u-translation-of", mdr_microformats, mp2_u_translation_of },
    { "u-uid", mdr_microformats, mp2_u_uid },
    { "u-url", mdr_microformats, mp2_u_url },
    { "u-video", mdr_microformats, mp2_u_video },
    { "u-watch-of", mdr_microformats, mp2_u_watch_of },

    { nullptr, mdr_none, sp_illegal } };

typedef ssc_mm < ::std::string, property_name* > mpn_t;
typedef ssc_map < e_schema_property, property_name* > mnp_t;
mpn_t mpn;
mnp_t mnp;

void schema_name_init (nitpick& nits)
{   DBG_ASSERT (mpn.empty ());
    DBG_ASSERT (mnp.empty ());
    for (property_name* p = &namtab [0]; p -> prop_ != sp_illegal; ++p)
        if (p -> name_ != nullptr)
        {   ::std::string n (::boost::to_lower_copy (::std::string (p -> name_)));
            for (mpn_t::const_iterator ci = mpn.find (n); (ci != mpn.cend ()) && compare_no_case (ci -> first, n); ++ci)
                if (ci -> second -> root_ == p -> root_)
                {   nits.pick (nit_symbol_aleady_defined, es_catastrophic, ec_init, "schema property name ", n, " (", p -> root_, ") defined more than once");
                    break; }
            auto i = mnp.find (p -> prop_);
            if (i != mnp.cend ())
            {   nits.pick (nit_symbol_aleady_defined, es_splurge, ec_init, "schema property ", p -> prop_, " ('", i -> second -> name_, "', '", p -> name_, "') has many names");
                mpn.insert (mpn_t::value_type (n, p)); }
            else
            {   mpn.insert (mpn_t::value_type (n, p));
                mnp.insert (mnp_t::value_type (p -> prop_, p)); } } }

::std::string schema_property_name (const e_schema_property p)
{   mnp_t::const_iterator i = mnp.find (p);
    if (i == mnp.cend ()) return ::std::string ();
    return i -> second -> name_; }

e_schema_property get_schema_property (const ::std::string& s)
{   mpn_t::const_iterator i = mpn.find (s);
    if (i == mpn.cend ()) return sp_illegal;
    return i -> second -> prop_; }

vsp_t get_schema_properties (const ::std::string& s, const e_microdata_root mdr)
{   vsp_t res;
    for (mpn_t::const_iterator i = mpn.find (s); (i != mpn.cend ()) && compare_no_case (i -> first, s); ++i)
        if ((mdr == mdr_none) || (mdr == i -> second -> root_))
            res.emplace_back (i -> second -> prop_);
    return res; }

e_microdata_root get_property_root (const ::std::string& s)
{   mpn_t::const_iterator ci = mpn.find (s);
    if (ci == mpn.cend ()) return mdr_none;
    return ci -> second -> root_; }

e_microdata_root get_property_root (const e_schema_property sp)
{   mnp_t::const_iterator ci = mnp.find (sp);
    if (ci == mnp.cend ()) return mdr_none;
    return ci -> second -> root_; }
